@@ -3,12 +3,12 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
+import type { ViewConfigurationChangedEvent, ViewLinesChangedEvent, ViewLinesDeletedEvent, ViewLinesInsertedEvent, ViewScrollChangedEvent, ViewTokensChangedEvent } from '../../common/viewEvents.js';
 import type { ViewportData } from '../../common/viewLayout/viewLinesViewportData.js';
 import type { ViewLineOptions } from '../viewParts/viewLines/viewLineOptions.js';
 
 export const enum BindingId {
-	GlyphInfo0,
-	GlyphInfo1,
+	GlyphInfo,
 	Cells,
 	TextureSampler,
 	Texture,
@@ -21,6 +21,13 @@ export interface IGpuRenderStrategy {
 	readonly wgsl: string;
 	readonly bindGroupEntries: GPUBindGroupEntry[];
 
+	onLinesDeleted(e: ViewLinesDeletedEvent): boolean;
+	onConfigurationChanged(e: ViewConfigurationChangedEvent): boolean;
+	onTokensChanged(e: ViewTokensChangedEvent): boolean;
+	onLinesInserted(e: ViewLinesInsertedEvent): boolean;
+	onLinesChanged(e: ViewLinesChangedEvent): boolean;
+	onScrollChanged(e?: ViewScrollChangedEvent): boolean;
+
 	/**
 	 * Resets the render strategy, clearing all data and setting up for a new frame.
 	 */
