@@ -259,6 +259,13 @@ export abstract class AbstractLogger extends Disposable implements ILogger {
 		return this.level !== LogLevel.Off && this.level <= level;
 	}
 
+	protected canLog(level: LogLevel): boolean {
+		if (this._store.isDisposed) {
+			return false;
+		}
+		return this.checkLogLevel(level);
+	}
+
 	abstract trace(message: string, ...args: any[]): void;
 	abstract debug(message: string, ...args: any[]): void;
 	abstract info(message: string, ...args: any[]): void;
@@ -269,8 +276,6 @@ export abstract class AbstractLogger extends Disposable implements ILogger {
 
 export abstract class AbstractMessageLogger extends AbstractLogger implements ILogger {
 
-	protected abstract log(level: LogLevel, message: string): void;
-
 	constructor(private readonly logAlways?: boolean) {
 		super();
 	}
@@ -280,32 +285,31 @@ export abstract class AbstractMessageLogger extends AbstractLogger implements IL
 	}
 
 	trace(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Trace)) {
+		if (this.canLog(LogLevel.Trace)) {
 			this.log(LogLevel.Trace, format([message, ...args], true));
 		}
 	}
 
 	debug(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Debug)) {
+		if (this.canLog(LogLevel.Debug)) {
 			this.log(LogLevel.Debug, format([message, ...args]));
 		}
 	}
 
 	info(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Info)) {
+		if (this.canLog(LogLevel.Info)) {
 			this.log(LogLevel.Info, format([message, ...args]));
 		}
 	}
 
 	warn(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Warning)) {
+		if (this.canLog(LogLevel.Warning)) {
 			this.log(LogLevel.Warning, format([message, ...args]));
 		}
 	}
 
 	error(message: string | Error, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Error)) {
-
+		if (this.canLog(LogLevel.Error)) {
 			if (message instanceof Error) {
 				const array = Array.prototype.slice.call(arguments) as any[];
 				array[0] = message.stack;
@@ -317,6 +321,8 @@ export abstract class AbstractMessageLogger extends AbstractLogger implements IL
 	}
 
 	flush(): void { }
+
+	protected abstract log(level: LogLevel, message: string): void;
 }
 
 
@@ -331,7 +337,7 @@ export class ConsoleMainLogger extends AbstractLogger implements ILogger {
 	}
 
 	trace(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Trace)) {
+		if (this.canLog(LogLevel.Trace)) {
 			if (this.useColors) {
 				console.log(`\x1b[90m[main ${now()}]\x1b[0m`, message, ...args);
 			} else {
@@ -341,7 +347,7 @@ export class ConsoleMainLogger extends AbstractLogger implements ILogger {
 	}
 
 	debug(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Debug)) {
+		if (this.canLog(LogLevel.Debug)) {
 			if (this.useColors) {
 				console.log(`\x1b[90m[main ${now()}]\x1b[0m`, message, ...args);
 			} else {
@@ -351,7 +357,7 @@ export class ConsoleMainLogger extends AbstractLogger implements ILogger {
 	}
 
 	info(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Info)) {
+		if (this.canLog(LogLevel.Info)) {
 			if (this.useColors) {
 				console.log(`\x1b[90m[main ${now()}]\x1b[0m`, message, ...args);
 			} else {
@@ -361,7 +367,7 @@ export class ConsoleMainLogger extends AbstractLogger implements ILogger {
 	}
 
 	warn(message: string | Error, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Warning)) {
+		if (this.canLog(LogLevel.Warning)) {
 			if (this.useColors) {
 				console.warn(`\x1b[93m[main ${now()}]\x1b[0m`, message, ...args);
 			} else {
@@ -371,7 +377,7 @@ export class ConsoleMainLogger extends AbstractLogger implements ILogger {
 	}
 
 	error(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Error)) {
+		if (this.canLog(LogLevel.Error)) {
 			if (this.useColors) {
 				console.error(`\x1b[91m[main ${now()}]\x1b[0m`, message, ...args);
 			} else {
@@ -394,7 +400,7 @@ export class ConsoleLogger extends AbstractLogger implements ILogger {
 	}
 
 	trace(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Trace)) {
+		if (this.canLog(LogLevel.Trace)) {
 			if (this.useColors) {
 				console.log('%cTRACE', 'color: #888', message, ...args);
 			} else {
@@ -404,7 +410,7 @@ export class ConsoleLogger extends AbstractLogger implements ILogger {
 	}
 
 	debug(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Debug)) {
+		if (this.canLog(LogLevel.Debug)) {
 			if (this.useColors) {
 				console.log('%cDEBUG', 'background: #eee; color: #888', message, ...args);
 			} else {
@@ -414,7 +420,7 @@ export class ConsoleLogger extends AbstractLogger implements ILogger {
 	}
 
 	info(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Info)) {
+		if (this.canLog(LogLevel.Info)) {
 			if (this.useColors) {
 				console.log('%c INFO', 'color: #33f', message, ...args);
 			} else {
@@ -424,9 +430,9 @@ export class ConsoleLogger extends AbstractLogger implements ILogger {
 	}
 
 	warn(message: string | Error, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Warning)) {
+		if (this.canLog(LogLevel.Warning)) {
 			if (this.useColors) {
-				console.log('%c WARN', 'color: #993', message, ...args);
+				console.warn('%c WARN', 'color: #993', message, ...args);
 			} else {
 				console.log(message, ...args);
 			}
@@ -434,9 +440,9 @@ export class ConsoleLogger extends AbstractLogger implements ILogger {
 	}
 
 	error(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Error)) {
+		if (this.canLog(LogLevel.Error)) {
 			if (this.useColors) {
-				console.log('%c  ERR', 'color: #f33', message, ...args);
+				console.error('%c  ERR', 'color: #f33', message, ...args);
 			} else {
 				console.error(message, ...args);
 			}
@@ -457,31 +463,31 @@ export class AdapterLogger extends AbstractLogger implements ILogger {
 	}
 
 	trace(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Trace)) {
+		if (this.canLog(LogLevel.Trace)) {
 			this.adapter.log(LogLevel.Trace, [this.extractMessage(message), ...args]);
 		}
 	}
 
 	debug(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Debug)) {
+		if (this.canLog(LogLevel.Debug)) {
 			this.adapter.log(LogLevel.Debug, [this.extractMessage(message), ...args]);
 		}
 	}
 
 	info(message: string, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Info)) {
+		if (this.canLog(LogLevel.Info)) {
 			this.adapter.log(LogLevel.Info, [this.extractMessage(message), ...args]);
 		}
 	}
 
 	warn(message: string | Error, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Warning)) {
+		if (this.canLog(LogLevel.Warning)) {
 			this.adapter.log(LogLevel.Warning, [this.extractMessage(message), ...args]);
 		}
 	}
 
 	error(message: string | Error, ...args: any[]): void {
-		if (this.checkLogLevel(LogLevel.Error)) {
+		if (this.canLog(LogLevel.Error)) {
 			this.adapter.log(LogLevel.Error, [this.extractMessage(message), ...args]);
 		}
 	}
@@ -491,7 +497,7 @@ export class AdapterLogger extends AbstractLogger implements ILogger {
 			return msg;
 		}
 
-		return toErrorMessage(msg, this.checkLogLevel(LogLevel.Trace));
+		return toErrorMessage(msg, this.canLog(LogLevel.Trace));
 	}
 
 	flush(): void {
