@@ -477,6 +477,11 @@ export interface IQuickPick<T extends IQuickPickItem, O extends { useSeparators:
 	 */
 	ok: boolean | 'default';
 
+	/**
+	 * The OK button label.
+	 */
+	okLabel: string | undefined;
+
 	/**
 	 * An event that is fired when the custom button is triggered. The custom button is a button with text rendered to the right of the input.
 	 */
