@@ -5,7 +5,7 @@
 
 import './media/statusbarpart.css';
 import { localize } from '../../../../nls.js';
-import { Disposable, DisposableStore, dispose, disposeIfDisposable, IDisposable, MutableDisposable, toDisposable } from '../../../../base/common/lifecycle.js';
+import { Disposable, DisposableStore, disposeIfDisposable, IDisposable, MutableDisposable, toDisposable } from '../../../../base/common/lifecycle.js';
 import { MultiWindowParts, Part } from '../../part.js';
 import { EventType as TouchEventType, Gesture, GestureEvent } from '../../../../base/browser/touch.js';
 import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
@@ -16,7 +16,8 @@ import { IThemeService } from '../../../../platform/theme/common/themeService.js
 import { STATUS_BAR_BACKGROUND, STATUS_BAR_FOREGROUND, STATUS_BAR_NO_FOLDER_BACKGROUND, STATUS_BAR_ITEM_HOVER_BACKGROUND, STATUS_BAR_BORDER, STATUS_BAR_NO_FOLDER_FOREGROUND, STATUS_BAR_NO_FOLDER_BORDER, STATUS_BAR_ITEM_COMPACT_HOVER_BACKGROUND, STATUS_BAR_ITEM_FOCUS_BORDER, STATUS_BAR_FOCUS_BORDER } from '../../../common/theme.js';
 import { IWorkspaceContextService, WorkbenchState } from '../../../../platform/workspace/common/workspace.js';
 import { contrastBorder, activeContrastBorder } from '../../../../platform/theme/common/colorRegistry.js';
-import { EventHelper, createStyleSheet, addDisposableListener, EventType, clearNode, getWindow } from '../../../../base/browser/dom.js';
+import { EventHelper, addDisposableListener, EventType, clearNode, getWindow } from '../../../../base/browser/dom.js';
+import { createStyleSheet } from '../../../../base/browser/domStylesheets.js';
 import { IStorageService } from '../../../../platform/storage/common/storage.js';
 import { Parts, IWorkbenchLayoutService } from '../../../services/layout/browser/layoutService.js';
 import { InstantiationType, registerSingleton } from '../../../../platform/instantiation/common/extensions.js';
@@ -74,6 +75,11 @@ export interface IStatusbarEntryContainer extends IDisposable {
 	 */
 	updateEntryVisibility(id: string, visible: boolean): void;
 
+	/**
+	 * Allows to override the appearance of an entry with the provided ID.
+	 */
+	overrideEntry(id: string, override: Partial<IStatusbarEntry>): IDisposable;
+
 	/**
 	 * Focused the status bar. If one of the status bar entries was focused, focuses it directly.
 	 */
@@ -133,6 +139,9 @@ class StatusbarPart extends Part implements IStatusbarEntryContainer {
 	private readonly _onWillDispose = this._register(new Emitter<void>());
 	readonly onWillDispose = this._onWillDispose.event;
 
+	private readonly onDidOverrideEntry = this._register(new Emitter<string>());
+	private readonly entryOverrides = new Map<string, Partial<IStatusbarEntry>>();
+
 	private leftItemsContainer: HTMLElement | undefined;
 	private rightItemsContainer: HTMLElement | undefined;
 
@@ -172,6 +181,28 @@ class StatusbarPart extends Part implements IStatusbarEntryContainer {
 		this._register(this.contextService.onDidChangeWorkbenchState(() => this.updateStyles()));
 	}
 
+	overrideEntry(id: string, override: Partial<IStatusbarEntry>): IDisposable {
+		this.entryOverrides.set(id, override);
+		this.onDidOverrideEntry.fire(id);
+
+		return toDisposable(() => {
+			const currentOverride = this.entryOverrides.get(id);
+			if (currentOverride === override) {
+				this.entryOverrides.delete(id);
+				this.onDidOverrideEntry.fire(id);
+			}
+		});
+	}
+
+	private withEntryOverride(entry: IStatusbarEntry, id: string): IStatusbarEntry {
+		const override = this.entryOverrides.get(id);
+		if (override) {
+			entry = { ...entry, ...override };
+		}
+
+		return entry;
+	}
+
 	addEntry(entry: IStatusbarEntry, id: string, alignment: StatusbarAlignment, priorityOrLocation: number | IStatusbarEntryLocation | IStatusbarEntryPriority = 0): IStatusbarEntryAccessor {
 		let priority: IStatusbarEntryPriority;
 		if (isStatusbarEntryPriority(priorityOrLocation)) {
@@ -219,10 +250,11 @@ class StatusbarPart extends Part implements IStatusbarEntryContainer {
 	}
 
 	private doAddEntry(entry: IStatusbarEntry, id: string, alignment: StatusbarAlignment, priority: IStatusbarEntryPriority): IStatusbarEntryAccessor {
+		const disposables = new DisposableStore();
 
 		// View model item
 		const itemContainer = this.doCreateStatusItem(id, alignment);
-		const item = this.instantiationService.createInstance(StatusbarEntryItem, itemContainer, entry, this.hoverDelegate);
+		const item = disposables.add(this.instantiationService.createInstance(StatusbarEntryItem, itemContainer, this.withEntryOverride(entry, id), this.hoverDelegate));
 
 		// View model entry
 		const viewModelEntry: IStatusbarViewModelEntry = new class implements IStatusbarViewModelEntry {
@@ -244,9 +276,11 @@ class StatusbarPart extends Part implements IStatusbarEntryContainer {
 			this.appendStatusbarEntry(viewModelEntry);
 		}
 
-		return {
+		let lastEntry = entry;
+		const accessor: IStatusbarEntryAccessor = {
 			update: entry => {
-				item.update(entry);
+				lastEntry = entry;
+				item.update(this.withEntryOverride(entry, id));
 			},
 			dispose: () => {
 				const { needsFullRefresh } = this.doAddOrRemoveModelEntry(viewModelEntry, false);
@@ -254,10 +288,20 @@ class StatusbarPart extends Part implements IStatusbarEntryContainer {
 					this.appendStatusbarEntries();
 				} else {
 					itemContainer.remove();
+					this.updateCompactEntries();
 				}
-				dispose(item);
+				disposables.dispose();
 			}
 		};
+
+		// React to overrides
+		disposables.add(this.onDidOverrideEntry.event(overrideEntryId => {
+			if (overrideEntryId === id) {
+				accessor.update(lastEntry);
+			}
+		}));
+
+		return accessor;
 	}
 
 	private doCreateStatusItem(id: string, alignment: StatusbarAlignment, ...extraClasses: string[]): HTMLElement {
@@ -789,6 +833,16 @@ export class StatusbarService extends MultiWindowParts<StatusbarPart> implements
 		}
 	}
 
+	overrideEntry(id: string, override: Partial<IStatusbarEntry>): IDisposable {
+		const disposables = new DisposableStore();
+
+		for (const part of this.parts) {
+			disposables.add(part.overrideEntry(id, override));
+		}
+
+		return disposables;
+	}
+
 	focus(preserveEntryFocus?: boolean): void {
 		this.activePart.focus(preserveEntryFocus);
 	}
@@ -855,6 +909,10 @@ export class ScopedStatusbarService extends Disposable implements IStatusbarServ
 		this.statusbarEntryContainer.updateEntryVisibility(id, visible);
 	}
 
+	overrideEntry(id: string, override: Partial<IStatusbarEntry>): IDisposable {
+		return this.statusbarEntryContainer.overrideEntry(id, override);
+	}
+
 	focus(preserveEntryFocus?: boolean): void {
 		this.statusbarEntryContainer.focus(preserveEntryFocus);
 	}
