@@ -7,48 +7,48 @@ import { toAction } from '../../../../../base/common/actions.js';
 import { coalesce } from '../../../../../base/common/arrays.js';
 import { Codicon } from '../../../../../base/common/codicons.js';
 import { fromNowByDay } from '../../../../../base/common/date.js';
+import { Event } from '../../../../../base/common/event.js';
 import { KeyCode, KeyMod } from '../../../../../base/common/keyCodes.js';
-import { DisposableStore } from '../../../../../base/common/lifecycle.js';
+import { Disposable, DisposableStore, markAsSingleton } from '../../../../../base/common/lifecycle.js';
 import { ThemeIcon } from '../../../../../base/common/themables.js';
+import { URI } from '../../../../../base/common/uri.js';
 import { ICodeEditor } from '../../../../../editor/browser/editorBrowser.js';
-import { EditorAction2, ServicesAccessor } from '../../../../../editor/browser/editorExtensions.js';
+import { EditorAction2 } from '../../../../../editor/browser/editorExtensions.js';
 import { Position } from '../../../../../editor/common/core/position.js';
 import { SuggestController } from '../../../../../editor/contrib/suggest/browser/suggestController.js';
-import { localize, localize2 } from '../../../../../nls.js';
+import { ILocalizedString, localize, localize2 } from '../../../../../nls.js';
 import { IActionViewItemService } from '../../../../../platform/actions/browser/actionViewItemService.js';
 import { DropdownWithPrimaryActionViewItem } from '../../../../../platform/actions/browser/dropdownWithPrimaryActionViewItem.js';
 import { Action2, MenuId, MenuItemAction, MenuRegistry, registerAction2, SubmenuItemAction } from '../../../../../platform/actions/common/actions.js';
-import { ContextKeyExpr } from '../../../../../platform/contextkey/common/contextkey.js';
+import { ContextKeyExpr, ContextKeyExpression, IContextKeyService } from '../../../../../platform/contextkey/common/contextkey.js';
 import { IsLinuxContext, IsWindowsContext } from '../../../../../platform/contextkey/common/contextkeys.js';
-import { IInstantiationService } from '../../../../../platform/instantiation/common/instantiation.js';
+import { IInstantiationService, ServicesAccessor } from '../../../../../platform/instantiation/common/instantiation.js';
 import { KeybindingWeight } from '../../../../../platform/keybinding/common/keybindingsRegistry.js';
 import { IOpenerService } from '../../../../../platform/opener/common/opener.js';
-import { IProductService } from '../../../../../platform/product/common/productService.js';
-import { ProgressLocation } from '../../../../../platform/progress/common/progress.js';
+import product from '../../../../../platform/product/common/product.js';
 import { IQuickInputButton, IQuickInputService, IQuickPickItem, IQuickPickSeparator } from '../../../../../platform/quickinput/common/quickInput.js';
 import { ToggleTitleBarConfigAction } from '../../../../browser/parts/titlebar/titlebarActions.js';
 import { IWorkbenchContribution } from '../../../../common/contributions.js';
 import { IEditorGroupsService } from '../../../../services/editor/common/editorGroupsService.js';
 import { ACTIVE_GROUP, IEditorService } from '../../../../services/editor/common/editorService.js';
+import { IHostService } from '../../../../services/host/browser/host.js';
 import { IViewsService } from '../../../../services/views/common/viewsService.js';
-import { IExtensionsWorkbenchService } from '../../../extensions/common/extensions.js';
+import { EXTENSIONS_CATEGORY, IExtensionsWorkbenchService } from '../../../extensions/common/extensions.js';
 import { ChatAgentLocation, IChatAgentService } from '../../common/chatAgents.js';
 import { ChatContextKeys } from '../../common/chatContextKeys.js';
 import { extractAgentAndCommand } from '../../common/chatParserTypes.js';
+import { IChatQuotasService, OPEN_CHAT_QUOTA_EXCEEDED_DIALOG, quotaToButtonMessage } from '../chatQuotasService.js';
 import { IChatDetail, IChatService } from '../../common/chatService.js';
+import { IChatVariablesService } from '../../common/chatVariables.js';
 import { IChatRequestViewModel, IChatResponseViewModel, isRequestVM } from '../../common/chatViewModel.js';
 import { IChatWidgetHistoryService } from '../../common/chatWidgetHistoryService.js';
-import { CHAT_VIEW_ID, IChatWidget, IChatWidgetService, showChatView } from '../chat.js';
+import { CopilotUsageExtensionFeatureId } from '../../common/languageModelStats.js';
+import { ChatViewId, IChatWidget, IChatWidgetService, showChatView } from '../chat.js';
 import { IChatEditorOptions } from '../chatEditor.js';
 import { ChatEditorInput } from '../chatEditorInput.js';
 import { ChatViewPane } from '../chatViewPane.js';
 import { convertBufferToScreenshotVariable } from '../contrib/screenshot.js';
 import { clearChatEditor } from './chatClear.js';
-import product from '../../../../../platform/product/common/product.js';
-import { URI } from '../../../../../base/common/uri.js';
-import { IHostService } from '../../../../services/host/browser/host.js';
-import { isCancellationError } from '../../../../../base/common/errors.js';
-import { ITelemetryService } from '../../../../../platform/telemetry/common/telemetry.js';
 
 export const CHAT_CATEGORY = localize2('chat.category', 'Chat');
 export const CHAT_OPEN_ACTION_ID = 'workbench.action.chat.open';
@@ -62,6 +62,10 @@ export interface IChatViewOpenOptions {
 	 * Whether the query is partial and will await more input from the user.
 	 */
 	isPartialQuery?: boolean;
+	/**
+	 * A list of simple variables that will be resolved and attached if they exist.
+	 */
+	variableIds?: string[];
 	/**
 	 * Any previous chat requests and responses that should be shown in the chat view.
 	 */
@@ -78,14 +82,6 @@ export interface IChatViewOpenRequestEntry {
 	response: string;
 }
 
-const defaultChat = {
-	extensionId: product.defaultChatAgent?.extensionId ?? '',
-	name: product.defaultChatAgent?.name ?? '',
-	icon: Codicon[product.defaultChatAgent?.icon as keyof typeof Codicon ?? 'commentDiscussion'],
-	documentationUrl: product.defaultChatAgent?.documentationUrl ?? '',
-	gettingStartedCommand: product.defaultChatAgent?.gettingStartedCommand ?? '',
-};
-
 class OpenChatGlobalAction extends Action2 {
 
 	static readonly TITLE = localize2('openChat', "Open Chat");
@@ -94,9 +90,12 @@ class OpenChatGlobalAction extends Action2 {
 		super({
 			id: CHAT_OPEN_ACTION_ID,
 			title: OpenChatGlobalAction.TITLE,
-			icon: defaultChat.icon,
+			icon: Codicon.copilot,
 			f1: true,
-			precondition: ChatContextKeys.panelParticipantRegistered,
+			precondition: ContextKeyExpr.or(
+				ChatContextKeys.Setup.installed,
+				ChatContextKeys.panelParticipantRegistered
+			),
 			category: CHAT_CATEGORY,
 			keybinding: {
 				weight: KeybindingWeight.WorkbenchContrib,
@@ -107,7 +106,7 @@ class OpenChatGlobalAction extends Action2 {
 			},
 			menu: {
 				id: MenuId.ChatCommandCenter,
-				group: 'a_chat',
+				group: 'a_open',
 				order: 1
 			}
 		});
@@ -117,8 +116,10 @@ class OpenChatGlobalAction extends Action2 {
 		opts = typeof opts === 'string' ? { query: opts } : opts;
 
 		const chatService = accessor.get(IChatService);
+		const chatVariablesService = accessor.get(IChatVariablesService);
 		const viewsService = accessor.get(IViewsService);
 		const hostService = accessor.get(IHostService);
+
 		const chatWidget = await showChatView(viewsService);
 		if (!chatWidget) {
 			return;
@@ -141,6 +142,21 @@ class OpenChatGlobalAction extends Action2 {
 				chatWidget.acceptInput(opts.query);
 			}
 		}
+		if (opts?.variableIds && opts.variableIds.length > 0) {
+			const actualVariables = chatVariablesService.getVariables(ChatAgentLocation.Panel);
+			for (const actualVariable of actualVariables) {
+				if (opts.variableIds.includes(actualVariable.id)) {
+					chatWidget.attachmentModel.addContext({
+						range: undefined,
+						id: actualVariable.id ?? '',
+						value: undefined,
+						fullName: actualVariable.fullName,
+						name: actualVariable.name,
+						icon: actualVariable.icon
+					});
+				}
+			}
+		}
 
 		chatWidget.focusInput();
 	}
@@ -153,7 +169,7 @@ class ChatHistoryAction extends Action2 {
 			title: localize2('chat.history.label', "Show Chats..."),
 			menu: {
 				id: MenuId.ViewTitle,
-				when: ContextKeyExpr.equals('view', CHAT_VIEW_ID),
+				when: ContextKeyExpr.equals('view', ChatViewId),
 				group: 'navigation',
 				order: 2
 			},
@@ -244,7 +260,7 @@ class ChatHistoryAction extends Action2 {
 				try {
 					const item = picker.selectedItems[0];
 					const sessionId = item.chat.sessionId;
-					const view = await viewsService.openView(CHAT_VIEW_ID) as ChatViewPane;
+					const view = await viewsService.openView(ChatViewId) as ChatViewPane;
 					view.loadSession(sessionId);
 				} finally {
 					picker.hide();
@@ -320,6 +336,14 @@ class ChatAddAction extends Action2 {
 	}
 }
 
+MenuRegistry.appendMenuItem(MenuId.ViewTitle, {
+	command: {
+		id: 'update.showCurrentReleaseNotes',
+		title: localize2('chat.releaseNotes.label', "Explore New Features"),
+	},
+	when: ContextKeyExpr.equals('view', ChatViewId)
+});
+
 export function registerChatActions() {
 	registerAction2(OpenChatGlobalAction);
 	registerAction2(ChatHistoryAction);
@@ -359,7 +383,7 @@ export function registerChatActions() {
 			const chatService = accessor.get(IChatService);
 			chatService.clearAllHistoryEntries();
 
-			const chatView = viewsService.getViewWithId(CHAT_VIEW_ID) as ChatViewPane | undefined;
+			const chatView = viewsService.getViewWithId(ChatViewId) as ChatViewPane | undefined;
 			if (chatView) {
 				chatView.widget.clear();
 			}
@@ -440,9 +464,52 @@ export function registerChatActions() {
 		}
 	});
 
-	registerAction2(InstallChatWithPromptAction);
-	registerAction2(InstallChatWithoutPromptAction);
-	registerAction2(LearnMoreChatAction);
+	function registerOpenLinkAction(id: string, title: ILocalizedString, url: string, order: number, contextKey: ContextKeyExpression = ChatContextKeys.enabled): void {
+		registerAction2(class extends Action2 {
+			constructor() {
+				super({
+					id,
+					title,
+					category: CHAT_CATEGORY,
+					f1: true,
+					precondition: contextKey,
+					menu: {
+						id: MenuId.ChatCommandCenter,
+						group: 'y_manage',
+						order,
+						when: contextKey
+					}
+				});
+			}
+
+			override async run(accessor: ServicesAccessor): Promise<void> {
+				const openerService = accessor.get(IOpenerService);
+				openerService.open(URI.parse(url));
+			}
+		});
+	}
+
+	const nonEnterpriseCopilotUsers = ContextKeyExpr.and(ChatContextKeys.enabled, ContextKeyExpr.notEquals('config.github.copilot.advanced.authProvider', 'github-enterprise'));
+	registerOpenLinkAction('workbench.action.chat.managePlan', localize2('managePlan', "Manage Copilot Plan"), defaultChat.managePlanUrl, 1, nonEnterpriseCopilotUsers);
+	registerOpenLinkAction('workbench.action.chat.manageSettings', localize2('manageSettings', "Manage Copilot Settings"), defaultChat.manageSettingsUrl, 2, nonEnterpriseCopilotUsers);
+	registerOpenLinkAction('workbench.action.chat.learnMore', localize2('learnMore', "Learn More"), defaultChat.documentationUrl, 3);
+
+	registerAction2(class ShowExtensionsUsingCopilit extends Action2 {
+
+		constructor() {
+			super({
+				id: 'workbench.action.chat.showExtensionsUsingCopilot',
+				title: localize2('showCopilotUsageExtensions', "Show Extensions using Copilot"),
+				f1: true,
+				category: EXTENSIONS_CATEGORY,
+			});
+		}
+
+		override async run(accessor: ServicesAccessor): Promise<void> {
+			const extensionsWorkbenchService = accessor.get(IExtensionsWorkbenchService);
+			extensionsWorkbenchService.openSearch(`@feature:${CopilotUsageExtensionFeatureId}`);
+		}
+	});
 }
 
 export function stringifyItem(item: IChatRequestViewModel | IChatResponseViewModel, includeName = true): string {
@@ -456,45 +523,53 @@ export function stringifyItem(item: IChatRequestViewModel | IChatResponseViewMod
 
 // --- command center chat
 
+const defaultChat = {
+	documentationUrl: product.defaultChatAgent?.documentationUrl ?? '',
+	manageSettingsUrl: product.defaultChatAgent?.manageSettingsUrl ?? '',
+	managePlanUrl: product.defaultChatAgent?.managePlanUrl ?? '',
+};
+
 MenuRegistry.appendMenuItem(MenuId.CommandCenter, {
 	submenu: MenuId.ChatCommandCenter,
 	title: localize('title4', "Chat"),
-	icon: defaultChat.icon,
+	icon: Codicon.copilot,
 	when: ContextKeyExpr.and(
-		ContextKeyExpr.has('config.chat.commandCenter.enabled'),
-		ContextKeyExpr.or(ChatContextKeys.panelParticipantRegistered, ChatContextKeys.installEntitled)
+		ChatContextKeys.supported,
+		ContextKeyExpr.has('config.chat.commandCenter.enabled')
 	),
 	order: 10001,
 });
 
-registerAction2(class ToggleChatControl extends ToggleTitleBarConfigAction {
+registerAction2(class ToggleCopilotControl extends ToggleTitleBarConfigAction {
 	constructor() {
 		super(
 			'chat.commandCenter.enabled',
-			localize('toggle.chatControl', 'Chat Controls'),
-			localize('toggle.chatControlsDescription', "Toggle visibility of the Chat Controls in title bar"), 3, false,
+			localize('toggle.chatControl', 'Copilot Controls'),
+			localize('toggle.chatControlsDescription', "Toggle visibility of the Copilot Controls in title bar"), 5, false,
 			ContextKeyExpr.and(
-				ContextKeyExpr.has('config.window.commandCenter'),
-				ContextKeyExpr.or(ChatContextKeys.panelParticipantRegistered, ChatContextKeys.installEntitled)
+				ChatContextKeys.supported,
+				ContextKeyExpr.has('config.window.commandCenter')
 			)
 		);
 	}
 });
 
-export class ChatCommandCenterRendering implements IWorkbenchContribution {
+export class ChatCommandCenterRendering extends Disposable implements IWorkbenchContribution {
 
 	static readonly ID = 'chat.commandCenterRendering';
 
-	private readonly _store = new DisposableStore();
-
 	constructor(
 		@IActionViewItemService actionViewItemService: IActionViewItemService,
 		@IChatAgentService agentService: IChatAgentService,
+		@IChatQuotasService chatQuotasService: IChatQuotasService,
 		@IInstantiationService instantiationService: IInstantiationService,
+		@IContextKeyService contextKeyService: IContextKeyService,
 	) {
+		super();
 
-		this._store.add(actionViewItemService.register(MenuId.CommandCenter, MenuId.ChatCommandCenter, (action, options) => {
+		const contextKeySet = new Set([ChatContextKeys.Setup.signedOut.key]);
 
+		const disposable = actionViewItemService.register(MenuId.CommandCenter, MenuId.ChatCommandCenter, (action, options) => {
 			if (!(action instanceof SubmenuItemAction)) {
 				return undefined;
 			}
@@ -506,144 +581,43 @@ export class ChatCommandCenterRendering implements IWorkbenchContribution {
 			});
 
 			const chatExtensionInstalled = agentService.getAgents().some(agent => agent.isDefault);
+			const { chatQuotaExceeded, completionsQuotaExceeded } = chatQuotasService.quotas;
+			const signedOut = contextKeyService.getContextKeyValue<boolean>(ChatContextKeys.Setup.signedOut.key) ?? false;
 
-			const primaryAction = instantiationService.createInstance(MenuItemAction, {
-				id: chatExtensionInstalled ? CHAT_OPEN_ACTION_ID : InstallChatWithPromptAction.ID,
-				title: chatExtensionInstalled ? OpenChatGlobalAction.TITLE : InstallChatWithPromptAction.TITLE,
-				icon: defaultChat.icon,
-			}, undefined, undefined, undefined, undefined);
-
-			return instantiationService.createInstance(
-				DropdownWithPrimaryActionViewItem,
-				primaryAction, dropdownAction, action.actions,
-				'',
-				{
-					...options,
-					skipTelemetry: true, // already handled by the workbench action bar
+			let primaryActionId: string;
+			let primaryActionTitle: string;
+			let primaryActionIcon: ThemeIcon;
+			if (!chatExtensionInstalled) {
+				primaryActionId = 'workbench.action.chat.triggerSetup';
+				primaryActionTitle = localize('triggerChatSetup', "Use AI Features with Copilot for Free...");
+				primaryActionIcon = Codicon.copilot;
+			} else {
+				if (signedOut) {
+					primaryActionId = CHAT_OPEN_ACTION_ID;
+					primaryActionTitle = localize('signInToChatSetup', "Sign in to Use Copilot...");
+					primaryActionIcon = Codicon.copilotWarning;
+				} else if (chatQuotaExceeded || completionsQuotaExceeded) {
+					primaryActionId = OPEN_CHAT_QUOTA_EXCEEDED_DIALOG;
+					primaryActionTitle = quotaToButtonMessage({ chatQuotaExceeded, completionsQuotaExceeded });
+					primaryActionIcon = Codicon.copilotWarning;
+				} else {
+					primaryActionId = CHAT_OPEN_ACTION_ID;
+					primaryActionTitle = OpenChatGlobalAction.TITLE.value;
+					primaryActionIcon = Codicon.copilot;
 				}
-			);
-
-		}, agentService.onDidChangeAgents));
-	}
-
-	dispose() {
-		this._store.dispose();
-	}
-}
-
-abstract class BaseInstallChatAction extends Action2 {
-
-	protected abstract getJustification(productService: IProductService): string | undefined;
-
-	override async run(accessor: ServicesAccessor): Promise<void> {
-		const extensionsWorkbenchService = accessor.get(IExtensionsWorkbenchService);
-		const productService = accessor.get(IProductService);
-		const telemetryService = accessor.get(ITelemetryService);
-
-		type InstallChatClassification = {
-			owner: 'bpasero';
-			comment: 'Provides insight into chat installation.';
-			installResult: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'Whether the extension was installed successfully, cancelled or failed to install.' };
-			hasJustification: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'The type of window error to understand the nature of the error better.' };
-		};
-		type InstallChatEvent = {
-			hasJustification: boolean;
-			installResult: 'installed' | 'cancelled' | 'failed';
-		};
-
-		const justification = this.getJustification(productService);
-
-		let installResult: 'installed' | 'cancelled' | 'failed';
-		try {
-			await extensionsWorkbenchService.install(defaultChat.extensionId, {
-				justification,
-				enable: true,
-				installPreReleaseVersion: productService.quality !== 'stable'
-			}, ProgressLocation.Notification);
-
-			installResult = 'installed';
-		} catch (error) {
-			installResult = isCancellationError(error) ? 'cancelled' : 'failed';
-		}
-
-		telemetryService.publicLog2<InstallChatEvent, InstallChatClassification>('commandCenter.chatInstall', {
-			installResult,
-			hasJustification: !!justification
-		});
-	}
-}
-
-class InstallChatWithPromptAction extends BaseInstallChatAction {
-
-	static readonly ID = 'workbench.action.chat.installWithPrompt';
-	static readonly TITLE = localize2('installChat', "Install {0}", defaultChat.name);
-
-	constructor() {
-		super({
-			id: InstallChatWithPromptAction.ID,
-			title: InstallChatWithPromptAction.TITLE,
-			icon: defaultChat.icon,
-			category: CHAT_CATEGORY
-		});
-	}
-
-	protected getJustification(productService: IProductService): string {
-		return localize('installChatGlobalAction.justification', "AI features in {0} require this extension. Your account already has access to {1}.", productService.nameShort, defaultChat.name);
-	}
-}
-
-class InstallChatWithoutPromptAction extends BaseInstallChatAction {
-
-	static readonly ID = 'workbench.action.chat.installWithoutPrompt';
-	static readonly TITLE = localize2('installChat', "Install {0}", defaultChat.name);
-
-	constructor() {
-		super({
-			id: InstallChatWithoutPromptAction.ID,
-			title: InstallChatWithoutPromptAction.TITLE,
-			category: CHAT_CATEGORY,
-			menu: {
-				id: MenuId.ChatCommandCenter,
-				group: 'a_atfirst',
-				order: 1,
-				when: ChatContextKeys.panelParticipantRegistered.negate()
 			}
-		});
-	}
+			return instantiationService.createInstance(DropdownWithPrimaryActionViewItem, instantiationService.createInstance(MenuItemAction, {
+				id: primaryActionId,
+				title: primaryActionTitle,
+				icon: primaryActionIcon,
+			}, undefined, undefined, undefined, undefined), dropdownAction, action.actions, '', { ...options, skipTelemetry: true });
+		}, Event.any(
+			agentService.onDidChangeAgents,
+			chatQuotasService.onDidChangeQuotas,
+			Event.filter(contextKeyService.onDidChangeContext, e => e.affectsSome(contextKeySet))
+		));
 
-	protected getJustification(): string | undefined {
-		return undefined;
-	}
-}
-
-class LearnMoreChatAction extends Action2 {
-
-	static readonly ID = 'workbench.action.chat.learnMore';
-	static readonly TITLE = localize2('learnMore', "Learn More");
-
-	constructor() {
-		super({
-			id: LearnMoreChatAction.ID,
-			title: LearnMoreChatAction.TITLE,
-			category: CHAT_CATEGORY,
-			menu: [{
-				id: MenuId.ChatCommandCenter,
-				group: 'a_atfirst',
-				order: 2,
-				when: ChatContextKeys.panelParticipantRegistered.negate()
-			}, {
-				id: MenuId.ChatCommandCenter,
-				group: 'z_atlast',
-				order: 1,
-				when: ChatContextKeys.panelParticipantRegistered
-			}]
-		});
-	}
-
-	override async run(accessor: ServicesAccessor): Promise<void> {
-		const openerService = accessor.get(IOpenerService);
-		if (defaultChat.documentationUrl) {
-			openerService.open(URI.parse(defaultChat.documentationUrl));
-		}
+		// Reduces flicker a bit on reload/restart
+		markAsSingleton(disposable);
 	}
 }
