@@ -7,16 +7,19 @@ import * as dom from '../../../../../base/browser/dom.js';
 import { StandardMouseEvent } from '../../../../../base/browser/mouseEvent.js';
 import { Codicon } from '../../../../../base/common/codicons.js';
 import { Emitter } from '../../../../../base/common/event.js';
-import { Disposable, IDisposable } from '../../../../../base/common/lifecycle.js';
+import { Disposable, DisposableStore, IDisposable } from '../../../../../base/common/lifecycle.js';
+import { autorun } from '../../../../../base/common/observable.js';
 import { equalsIgnoreCase } from '../../../../../base/common/strings.js';
 import { ThemeIcon } from '../../../../../base/common/themables.js';
 import { URI } from '../../../../../base/common/uri.js';
 import { MarkdownRenderer } from '../../../../../editor/browser/widget/markdownRenderer/browser/markdownRenderer.js';
 import { Range } from '../../../../../editor/common/core/range.js';
 import { ILanguageService } from '../../../../../editor/common/languages/language.js';
+import { ITextModel } from '../../../../../editor/common/model.js';
 import { getIconClasses } from '../../../../../editor/common/services/getIconClasses.js';
 import { IModelService } from '../../../../../editor/common/services/model.js';
-import { IResolvedTextEditorModel, ITextModelService } from '../../../../../editor/common/services/resolverService.js';
+import { ITextModelService } from '../../../../../editor/common/services/resolverService.js';
+import { localize } from '../../../../../nls.js';
 import { getFlatContextMenuActions } from '../../../../../platform/actions/browser/menuEntryActionViewItem.js';
 import { IMenuService, MenuId } from '../../../../../platform/actions/common/actions.js';
 import { IContextKeyService } from '../../../../../platform/contextkey/common/contextkey.js';
@@ -26,6 +29,7 @@ import { IInstantiationService } from '../../../../../platform/instantiation/com
 import { ILabelService } from '../../../../../platform/label/common/label.js';
 import { IEditorService } from '../../../../services/editor/common/editorService.js';
 import { IMarkdownVulnerability } from '../../common/annotations.js';
+import { IChatEditingService } from '../../common/chatEditingService.js';
 import { IChatProgressRenderableResponseContent } from '../../common/chatModel.js';
 import { IChatMarkdownContent } from '../../common/chatService.js';
 import { isRequestVM, isResponseVM } from '../../common/chatViewModel.js';
@@ -72,18 +76,23 @@ export class ChatMarkdownContentPart extends Disposable implements IChatContentP
 
 		// We release editors in order so that it's more likely that the same editor will be assigned if this element is re-rendered right away, like it often is during progressive rendering
 		const orderedDisposablesList: IDisposable[] = [];
-		let codeBlockIndex = codeBlockStartIndex;
+
+		// Need to track the index of the codeblock within the response so it can have a unique ID,
+		// and within this part to find it within the codeblocks array
+		let globalCodeBlockIndexStart = codeBlockStartIndex;
+		let thisPartCodeBlockIndexStart = 0;
 		const result = this._register(renderer.render(markdown.content, {
 			fillInIncompleteTokens,
 			codeBlockRendererSync: (languageId, text, raw) => {
-				const isCodeBlockComplete = !isResponseVM(context.element) || context.element.isComplete || !raw || raw?.endsWith('```');
+				const isCodeBlockComplete = !isResponseVM(context.element) || context.element.isComplete || !raw || raw?.trim().endsWith('```');
 				if ((!text || (text.startsWith('<vscode_codeblock_uri>') && !text.includes('\n'))) && !isCodeBlockComplete && rendererOptions.renderCodeBlockPills) {
 					const hideEmptyCodeblock = $('div');
 					hideEmptyCodeblock.style.display = 'none';
 					return hideEmptyCodeblock;
 				}
-				const index = codeBlockIndex++;
-				let textModel: Promise<IResolvedTextEditorModel>;
+				const globalIndex = globalCodeBlockIndexStart++;
+				const thisPartIndex = thisPartCodeBlockIndexStart++;
+				let textModel: Promise<ITextModel>;
 				let range: Range | undefined;
 				let vulns: readonly IMarkdownVulnerability[] | undefined;
 				let codemapperUri: URI | undefined;
@@ -91,24 +100,24 @@ export class ChatMarkdownContentPart extends Disposable implements IChatContentP
 					try {
 						const parsedBody = parseLocalFileData(text);
 						range = parsedBody.range && Range.lift(parsedBody.range);
-						textModel = this.textModelService.createModelReference(parsedBody.uri).then(ref => ref.object);
+						textModel = this.textModelService.createModelReference(parsedBody.uri).then(ref => ref.object.textEditorModel);
 					} catch (e) {
 						return $('div');
 					}
 				} else {
 					const sessionId = isResponseVM(element) || isRequestVM(element) ? element.sessionId : '';
-					const modelEntry = this.codeBlockModelCollection.getOrCreate(sessionId, element, index);
-					const fastUpdateModelEntry = this.codeBlockModelCollection.updateSync(sessionId, element, index, { text, languageId });
+					const modelEntry = this.codeBlockModelCollection.getOrCreate(sessionId, element, globalIndex);
+					const fastUpdateModelEntry = this.codeBlockModelCollection.updateSync(sessionId, element, globalIndex, { text, languageId, isComplete: isCodeBlockComplete });
 					vulns = modelEntry.vulns;
 					codemapperUri = fastUpdateModelEntry.codemapperUri;
 					textModel = modelEntry.model;
 				}
 
 				const hideToolbar = isResponseVM(element) && element.errorDetails?.responseIsFiltered;
-				const codeBlockInfo = { languageId, textModel, codeBlockIndex: index, element, range, hideToolbar, parentContextKeyService: contextKeyService, vulns, codemapperUri };
+				const codeBlockInfo: ICodeBlockData = { languageId, textModel, codeBlockIndex: globalIndex, codeBlockPartIndex: thisPartIndex, element, range, hideToolbar, parentContextKeyService: contextKeyService, vulns, codemapperUri };
 
 				if (!rendererOptions.renderCodeBlockPills || element.isCompleteAddedRequest || !codemapperUri) {
-					const ref = this.renderCodeBlock(codeBlockInfo, text, currentWidth, rendererOptions.editableCodeBlock);
+					const ref = this.renderCodeBlock(codeBlockInfo, text, isCodeBlockComplete, currentWidth);
 					this.allRefs.push(ref);
 
 					// Attach this after updating text/layout of the editor, so it should only be fired when the size updates later (horizontal scrollbar, wrapping)
@@ -118,7 +127,7 @@ export class ChatMarkdownContentPart extends Disposable implements IChatContentP
 					const ownerMarkdownPartId = this.id;
 					const info: IChatCodeBlockInfo = new class {
 						readonly ownerMarkdownPartId = ownerMarkdownPartId;
-						readonly codeBlockIndex = index;
+						readonly codeBlockIndex = globalIndex;
 						readonly element = element;
 						readonly isStreaming = !rendererOptions.renderCodeBlockPills;
 						codemapperUri = undefined; // will be set async
@@ -127,6 +136,7 @@ export class ChatMarkdownContentPart extends Disposable implements IChatContentP
 							// async and the uri might be undefined when it's read immediately
 							return ref.object.uri;
 						}
+						readonly uriPromise = textModel.then(model => model.uri);
 						public focus() {
 							ref.object.focus();
 						}
@@ -139,13 +149,12 @@ export class ChatMarkdownContentPart extends Disposable implements IChatContentP
 					return ref.object.element;
 				} else {
 					const requestId = isRequestVM(element) ? element.id : element.requestId;
-					const isStreaming = isResponseVM(element) ? !element.isComplete : !isCodeBlockComplete;
-					const ref = this.renderCodeBlockPill(element.sessionId, requestId, codeBlockInfo.codemapperUri, !isStreaming);
+					const ref = this.renderCodeBlockPill(element.sessionId, requestId, codeBlockInfo.codemapperUri, !isCodeBlockComplete);
 					if (isResponseVM(codeBlockInfo.element)) {
 						// TODO@joyceerhl: remove this code when we change the codeblockUri API to make the URI available synchronously
-						this.codeBlockModelCollection.update(codeBlockInfo.element.sessionId, codeBlockInfo.element, codeBlockInfo.codeBlockIndex, { text, languageId: codeBlockInfo.languageId }).then((e) => {
+						this.codeBlockModelCollection.update(codeBlockInfo.element.sessionId, codeBlockInfo.element, codeBlockInfo.codeBlockIndex, { text, languageId: codeBlockInfo.languageId, isComplete: isCodeBlockComplete }).then((e) => {
 							// Update the existing object's codemapperUri
-							this.codeblocks[codeBlockInfo.codeBlockIndex].codemapperUri = e.codemapperUri;
+							this.codeblocks[codeBlockInfo.codeBlockPartIndex].codemapperUri = e.codemapperUri;
 							this._onDidChangeHeight.fire();
 						});
 					}
@@ -153,13 +162,14 @@ export class ChatMarkdownContentPart extends Disposable implements IChatContentP
 					const ownerMarkdownPartId = this.id;
 					const info: IChatCodeBlockInfo = new class {
 						readonly ownerMarkdownPartId = ownerMarkdownPartId;
-						readonly codeBlockIndex = index;
+						readonly codeBlockIndex = globalIndex;
 						readonly element = element;
-						readonly isStreaming = isStreaming;
+						readonly isStreaming = !isCodeBlockComplete;
 						readonly codemapperUri = codemapperUri;
 						public get uri() {
 							return undefined;
 						}
+						readonly uriPromise = Promise.resolve(undefined);
 						public focus() {
 							return ref.object.element.focus();
 						}
@@ -182,10 +192,10 @@ export class ChatMarkdownContentPart extends Disposable implements IChatContentP
 		this.domNode = result.element;
 	}
 
-	private renderCodeBlockPill(sessionId: string, requestId: string, codemapperUri: URI | undefined, isCodeBlockComplete?: boolean): IDisposableReference<CollapsedCodeBlock> {
+	private renderCodeBlockPill(sessionId: string, requestId: string, codemapperUri: URI | undefined, isStreaming: boolean): IDisposableReference<CollapsedCodeBlock> {
 		const codeBlock = this.instantiationService.createInstance(CollapsedCodeBlock, sessionId, requestId);
 		if (codemapperUri) {
-			codeBlock.render(codemapperUri, !isCodeBlockComplete);
+			codeBlock.render(codemapperUri, isStreaming);
 		}
 		return {
 			object: codeBlock,
@@ -194,18 +204,18 @@ export class ChatMarkdownContentPart extends Disposable implements IChatContentP
 		};
 	}
 
-	private renderCodeBlock(data: ICodeBlockData, text: string, currentWidth: number, editableCodeBlock: boolean | undefined): IDisposableReference<CodeBlockPart> {
+	private renderCodeBlock(data: ICodeBlockData, text: string, isComplete: boolean, currentWidth: number): IDisposableReference<CodeBlockPart> {
 		const ref = this.editorPool.get();
 		const editorInfo = ref.object;
 		if (isResponseVM(data.element)) {
-			this.codeBlockModelCollection.update(data.element.sessionId, data.element, data.codeBlockIndex, { text, languageId: data.languageId }).then((e) => {
+			this.codeBlockModelCollection.update(data.element.sessionId, data.element, data.codeBlockIndex, { text, languageId: data.languageId, isComplete }).then((e) => {
 				// Update the existing object's codemapperUri
-				this.codeblocks[data.codeBlockIndex].codemapperUri = e.codemapperUri;
+				this.codeblocks[data.codeBlockPartIndex].codemapperUri = e.codemapperUri;
 				this._onDidChangeHeight.fire();
 			});
 		}
 
-		editorInfo.render(data, currentWidth, editableCodeBlock);
+		editorInfo.render(data, currentWidth);
 
 		return ref;
 	}
@@ -277,7 +287,7 @@ class CollapsedCodeBlock extends Disposable {
 		return this._uri;
 	}
 
-	private isStreaming: boolean | undefined;
+	private readonly _progressStore = this._store.add(new DisposableStore());
 
 	constructor(
 		sessionId: string,
@@ -289,6 +299,7 @@ class CollapsedCodeBlock extends Disposable {
 		@IContextMenuService private readonly contextMenuService: IContextMenuService,
 		@IContextKeyService private readonly contextKeyService: IContextKeyService,
 		@IMenuService private readonly menuService: IMenuService,
+		@IChatEditingService private readonly chatEditingService: IChatEditingService,
 	) {
 		super();
 		this.element = $('.chat-codeblock-pill-widget');
@@ -313,18 +324,17 @@ class CollapsedCodeBlock extends Disposable {
 		}));
 	}
 
-	render(uri: URI, isStreaming?: boolean) {
-		if (this.uri?.toString() === uri.toString() && this.isStreaming === isStreaming) {
-			return;
-		}
+	render(uri: URI, isStreaming?: boolean): void {
+		this._progressStore.clear();
 
 		this._uri = uri;
-		this.isStreaming = isStreaming;
 
 		const iconText = this.labelService.getUriBasenameLabel(uri);
+		const modifiedEntry = this.chatEditingService.currentEditingSession?.getEntry(uri);
+		const isComplete = !modifiedEntry?.isCurrentlyBeingModified.get();
 
 		let iconClasses: string[] = [];
-		if (isStreaming) {
+		if (isStreaming || !isComplete) {
 			const codicon = ThemeIcon.modify(Codicon.loading, 'spin');
 			iconClasses = ThemeIcon.asClassNameArray(codicon);
 		} else {
@@ -334,6 +344,51 @@ class CollapsedCodeBlock extends Disposable {
 
 		const iconEl = dom.$('span.icon');
 		iconEl.classList.add(...iconClasses);
-		this.element.replaceChildren(iconEl, dom.$('span.icon-label', {}, iconText));
+
+		const children = [dom.$('span.icon-label', {}, iconText)];
+		if (isStreaming) {
+			children.push(dom.$('span.label-detail', {}, localize('chat.codeblock.generating', "Generating edits...")));
+		} else if (!isComplete) {
+			children.push(dom.$('span.label-detail', {}, ''));
+		}
+		this.element.replaceChildren(iconEl, ...children);
+		this.element.title = this.labelService.getUriLabel(uri, { relative: false });
+
+		// Show a percentage progress that is driven by the rewrite
+
+		this._progressStore.add(autorun(r => {
+			const rewriteRatio = modifiedEntry?.rewriteRatio.read(r);
+
+			const labelDetail = this.element.querySelector('.label-detail');
+			const isComplete = !modifiedEntry?.isCurrentlyBeingModified.read(r);
+			if (labelDetail && !isStreaming && !isComplete) {
+				const value = rewriteRatio;
+				labelDetail.textContent = value === 0 || !value ? localize('chat.codeblock.applying', "Applying edits...") : localize('chat.codeblock.applyingPercentage', "Applying edits ({0}%)...", Math.round(value * 100));
+			} else if (labelDetail && !isStreaming && isComplete) {
+				iconEl.classList.remove(...iconClasses);
+				const fileKind = uri.path.endsWith('/') ? FileKind.FOLDER : FileKind.FILE;
+				iconEl.classList.add(...getIconClasses(this.modelService, this.languageService, uri, fileKind));
+				labelDetail.textContent = '';
+			}
+
+			if (!isStreaming && isComplete) {
+				const labelAdded = this.element.querySelector('.label-added') ?? this.element.appendChild(dom.$('span.label-added'));
+				const labelRemoved = this.element.querySelector('.label-removed') ?? this.element.appendChild(dom.$('span.label-removed'));
+				const changes = modifiedEntry?.diffInfo.read(r);
+				if (changes && !changes?.identical && !changes?.quitEarly) {
+					let removedLines = 0;
+					let addedLines = 0;
+					for (const change of changes.changes) {
+						removedLines += change.original.endLineNumberExclusive - change.original.startLineNumber;
+						addedLines += change.modified.endLineNumberExclusive - change.modified.startLineNumber;
+					}
+					labelAdded.textContent = `+${addedLines}`;
+					labelRemoved.textContent = `-${removedLines}`;
+					const insertionsFragment = addedLines === 1 ? localize('chat.codeblock.insertions.one', "1 insertion") : localize('chat.codeblock.insertions', "{0} insertions", addedLines);
+					const deletionsFragment = removedLines === 1 ? localize('chat.codeblock.deletions.one', "1 deletion") : localize('chat.codeblock.deletions', "{0} deletions", removedLines);
+					this.element.ariaLabel = this.element.title = localize('summary', 'Edited {0}, {1}, {2}', iconText, insertionsFragment, deletionsFragment);
+				}
+			}
+		}));
 	}
 }
