@@ -19,7 +19,8 @@ import { localize, localize2 } from '../../../../../nls.js';
 import { getFlatContextMenuActions } from '../../../../../platform/actions/browser/menuEntryActionViewItem.js';
 import { MenuWorkbenchToolBar } from '../../../../../platform/actions/browser/toolbar.js';
 import { Action2, IMenuService, MenuId, registerAction2 } from '../../../../../platform/actions/common/actions.js';
-import { IContextKeyService } from '../../../../../platform/contextkey/common/contextkey.js';
+import { IClipboardService } from '../../../../../platform/clipboard/common/clipboardService.js';
+import { ContextKeyExpr, IContextKeyService } from '../../../../../platform/contextkey/common/contextkey.js';
 import { IContextMenuService } from '../../../../../platform/contextview/browser/contextView.js';
 import { FileKind } from '../../../../../platform/files/common/files.js';
 import { IInstantiationService, ServicesAccessor } from '../../../../../platform/instantiation/common/instantiation.js';
@@ -42,13 +43,15 @@ import { IChatVariablesService } from '../../common/chatVariables.js';
 import { IChatRendererContent, IChatResponseViewModel } from '../../common/chatViewModel.js';
 import { ChatTreeItem, IChatWidgetService } from '../chat.js';
 import { IDisposableReference, ResourcePool } from './chatCollections.js';
-import { IChatContentPart } from './chatContentParts.js';
+import { IChatContentPart, IChatContentPartRenderContext } from './chatContentParts.js';
 
 const $ = dom.$;
 
 export interface IChatReferenceListItem extends IChatContentReference {
 	title?: string;
+	description?: string;
 	state?: WorkingSetEntryState;
+	excluded?: boolean;
 }
 
 export type IChatCollapsibleListItem = IChatReferenceListItem | IChatWarningMessage;
@@ -59,10 +62,12 @@ export class ChatCollapsibleListContentPart extends Disposable implements IChatC
 	private readonly _onDidChangeHeight = this._register(new Emitter<void>());
 	public readonly onDidChangeHeight = this._onDidChangeHeight.event;
 
+	private readonly hasFollowingContent: boolean;
+
 	constructor(
 		private readonly data: ReadonlyArray<IChatCollapsibleListItem>,
 		labelOverride: string | undefined,
-		element: IChatResponseViewModel,
+		context: IChatContentPartRenderContext,
 		contentReferencesListPool: CollapsibleListPool,
 		@IOpenerService openerService: IOpenerService,
 		@IMenuService menuService: IMenuService,
@@ -71,6 +76,8 @@ export class ChatCollapsibleListContentPart extends Disposable implements IChatC
 	) {
 		super();
 
+		const element = context.element as IChatResponseViewModel;
+		this.hasFollowingContent = context.contentIndex + 1 < context.content.length;
 		const referencesLabel = labelOverride ?? (data.length > 1 ?
 			localize('usedReferencesPlural', "Used {0} references", data.length) :
 			localize('usedReferencesSingular', "Used {0} reference", 1));
@@ -162,8 +169,7 @@ export class ChatCollapsibleListContentPart extends Disposable implements IChatC
 	}
 
 	hasSameContent(other: IChatRendererContent, followingContent: IChatRendererContent[], element: ChatTreeItem): boolean {
-		return other.kind === 'references' && other.references.length === this.data.length ||
-			other.kind === 'codeCitations' && other.citations.length === this.data.length;
+		return other.kind === 'references' && other.references.length === this.data.length && (!!followingContent.length === this.hasFollowingContent);
 	}
 
 	private updateAriaLabel(element: HTMLElement, label: string, expanded?: boolean): void {
@@ -370,33 +376,36 @@ class CollapsibleListRenderer implements IListRenderer<IChatCollapsibleListItem,
 		} else {
 			const uri = 'uri' in reference ? reference.uri : reference;
 			arg = uri;
+			const extraClasses = data.excluded ? ['excluded'] : [];
 			if (uri.scheme === 'https' && isEqualAuthority(uri.authority, 'github.com') && uri.path.includes('/tree/')) {
 				// Parse a nicer label for GitHub URIs that point at a particular commit + file
 				const label = uri.path.split('/').slice(1, 3).join('/');
 				const description = uri.path.split('/').slice(5).join('/');
-				templateData.label.setResource({ resource: uri, name: label, description }, { icon: Codicon.github, title: data.title });
+				templateData.label.setResource({ resource: uri, name: label, description }, { icon: Codicon.github, title: data.title, strikethrough: data.excluded, extraClasses });
 			} else if (uri.scheme === this.productService.urlProtocol && isEqualAuthority(uri.authority, SETTINGS_AUTHORITY)) {
 				// a nicer label for settings URIs
 				const settingId = uri.path.substring(1);
-				templateData.label.setResource({ resource: uri, name: settingId }, { icon: Codicon.settingsGear, title: localize('setting.hover', "Open setting '{0}'", settingId) });
+				templateData.label.setResource({ resource: uri, name: settingId }, { icon: Codicon.settingsGear, title: localize('setting.hover', "Open setting '{0}'", settingId), strikethrough: data.excluded, extraClasses });
 			} else if (matchesSomeScheme(uri, Schemas.mailto, Schemas.http, Schemas.https)) {
-				templateData.label.setResource({ resource: uri, name: uri.toString() }, { icon: icon ?? Codicon.globe, title: data.options?.status?.description ?? data.title ?? uri.toString() });
+				templateData.label.setResource({ resource: uri, name: uri.toString() }, { icon: icon ?? Codicon.globe, title: data.options?.status?.description ?? data.title ?? uri.toString(), strikethrough: data.excluded, extraClasses });
 			} else {
-				if (data.state === WorkingSetEntryState.Transient) {
+				if (data.state === WorkingSetEntryState.Transient || data.state === WorkingSetEntryState.Suggested) {
 					templateData.label.setResource(
 						{
 							resource: uri,
 							name: basenameOrAuthority(uri),
-							description: localize('chat.openEditor', 'Open Editor'),
+							description: data.description ?? localize('chat.openEditor', 'Open Editor'),
 							range: 'range' in reference ? reference.range : undefined,
-						}, { icon, title: data.options?.status?.description ?? data.title });
+						}, { icon, title: data.options?.status?.description ?? data.title, italic: data.state === WorkingSetEntryState.Suggested, strikethrough: data.excluded, extraClasses });
 				} else {
 					templateData.label.setFile(uri, {
 						fileKind: FileKind.FILE,
 						// Should not have this live-updating data on a historical reference
 						fileDecorations: undefined,
 						range: 'range' in reference ? reference.range : undefined,
-						title: data.options?.status?.description ?? data.title
+						title: data.options?.status?.description ?? data.title,
+						strikethrough: data.excluded,
+						extraClasses
 					});
 				}
 			}
@@ -468,7 +477,7 @@ registerAction2(class AddToChatAction extends Action2 {
 				id: MenuId.ChatAttachmentsContext,
 				group: 'chat',
 				order: 1,
-				when: ExplorerFolderContext.negate(),
+				when: ContextKeyExpr.and(ResourceContextKey.IsFileSystemResource, ExplorerFolderContext.negate()),
 			}]
 		});
 	}
@@ -490,4 +499,29 @@ registerAction2(class AddToChatAction extends Action2 {
 	}
 });
 
+registerAction2(class OpenChatReferenceLinkAction extends Action2 {
+
+	static readonly id = 'workbench.action.chat.copyLink';
+
+	constructor() {
+		super({
+			id: OpenChatReferenceLinkAction.id,
+			title: {
+				...localize2('copyLink', "Copy Link"),
+			},
+			f1: false,
+			menu: [{
+				id: MenuId.ChatAttachmentsContext,
+				group: 'chat',
+				order: 0,
+				when: ContextKeyExpr.or(ResourceContextKey.Scheme.isEqualTo(Schemas.http), ResourceContextKey.Scheme.isEqualTo(Schemas.https)),
+			}]
+		});
+	}
+
+	override async run(accessor: ServicesAccessor, resource: URI): Promise<void> {
+		await accessor.get(IClipboardService).writeResources([resource]);
+	}
+});
+
 //#endregion
