@@ -3,43 +3,45 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
-import { compareBy, delta } from '../../../../../base/common/arrays.js';
+import { coalesce, compareBy, delta } from '../../../../../base/common/arrays.js';
 import { AsyncIterableSource } from '../../../../../base/common/async.js';
 import { CancellationToken, CancellationTokenSource } from '../../../../../base/common/cancellation.js';
 import { Codicon } from '../../../../../base/common/codicons.js';
 import { BugIndicatingError } from '../../../../../base/common/errors.js';
 import { Emitter, Event } from '../../../../../base/common/event.js';
-import { Disposable, DisposableStore, IDisposable } from '../../../../../base/common/lifecycle.js';
+import { Disposable, DisposableStore, IDisposable, toDisposable } from '../../../../../base/common/lifecycle.js';
 import { ResourceMap } from '../../../../../base/common/map.js';
 import { derived, IObservable, observableValue, runOnChange, ValueWithChangeEventFromObservable } from '../../../../../base/common/observable.js';
 import { compare } from '../../../../../base/common/strings.js';
 import { ThemeIcon } from '../../../../../base/common/themables.js';
+import { isString } from '../../../../../base/common/types.js';
 import { URI } from '../../../../../base/common/uri.js';
 import { TextEdit } from '../../../../../editor/common/languages.js';
 import { ITextModelService } from '../../../../../editor/common/services/resolverService.js';
 import { localize, localize2 } from '../../../../../nls.js';
 import { IContextKey, IContextKeyService } from '../../../../../platform/contextkey/common/contextkey.js';
-import { EditorActivation } from '../../../../../platform/editor/common/editor.js';
 import { IFileService } from '../../../../../platform/files/common/files.js';
 import { IInstantiationService } from '../../../../../platform/instantiation/common/instantiation.js';
+import { ILogService } from '../../../../../platform/log/common/log.js';
 import { bindContextKey } from '../../../../../platform/observable/common/platformObservableUtils.js';
 import { IProgressService, ProgressLocation } from '../../../../../platform/progress/common/progress.js';
-import { EditorInput } from '../../../../common/editor/editorInput.js';
+import { IStorageService, StorageScope, StorageTarget } from '../../../../../platform/storage/common/storage.js';
 import { IWorkbenchAssignmentService } from '../../../../services/assignment/common/assignmentService.js';
 import { IDecorationData, IDecorationsProvider, IDecorationsService } from '../../../../services/decorations/common/decorations.js';
-import { IEditorGroup, IEditorGroupsService } from '../../../../services/editor/common/editorGroupsService.js';
 import { IEditorService } from '../../../../services/editor/common/editorService.js';
-import { MultiDiffEditor } from '../../../multiDiffEditor/browser/multiDiffEditor.js';
-import { MultiDiffEditorInput } from '../../../multiDiffEditor/browser/multiDiffEditorInput.js';
+import { ILifecycleService } from '../../../../services/lifecycle/common/lifecycle.js';
 import { IMultiDiffSourceResolver, IMultiDiffSourceResolverService, IResolvedMultiDiffSource, MultiDiffEditorItem } from '../../../multiDiffEditor/browser/multiDiffSourceResolverService.js';
-import { ICodeMapperResponse, ICodeMapperService } from '../../common/chatCodeMapperService.js';
+import { ChatAgentLocation, IChatAgentService } from '../../common/chatAgents.js';
 import { ChatContextKeys } from '../../common/chatContextKeys.js';
-import { applyingChatEditsContextKey, applyingChatEditsFailedContextKey, CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME, chatEditingMaxFileAssignmentName, chatEditingResourceContextKey, ChatEditingSessionState, decidedChatEditingResourceContextKey, defaultChatEditingMaxFileLimit, hasAppliedChatEditsContextKey, hasUndecidedChatEditingResourceContextKey, IChatEditingService, IChatEditingSession, IChatEditingSessionStream, inChatEditingSessionContextKey, WorkingSetEntryState } from '../../common/chatEditingService.js';
+import { applyingChatEditsContextKey, applyingChatEditsFailedContextKey, CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME, chatEditingMaxFileAssignmentName, chatEditingResourceContextKey, ChatEditingSessionState, decidedChatEditingResourceContextKey, defaultChatEditingMaxFileLimit, hasAppliedChatEditsContextKey, hasUndecidedChatEditingResourceContextKey, IChatEditingService, IChatEditingSession, IChatEditingSessionStream, IChatRelatedFile, IChatRelatedFilesProvider, IModifiedFileEntry, inChatEditingSessionContextKey, WorkingSetEntryState } from '../../common/chatEditingService.js';
 import { IChatResponseModel, IChatTextEditGroup } from '../../common/chatModel.js';
 import { IChatService } from '../../common/chatService.js';
 import { ChatEditingSession } from './chatEditingSession.js';
 import { ChatEditingSnapshotTextModelContentProvider, ChatEditingTextModelContentProvider } from './chatEditingTextModelContentProviders.js';
 
+
+const STORAGE_KEY_EDITING_SESSION = 'chat.editingSession';
+
 export class ChatEditingService extends Disposable implements IChatEditingService {
 
 	_serviceBrand: undefined;
@@ -60,11 +62,6 @@ export class ChatEditingService extends Disposable implements IChatEditingServic
 		return this._currentSessionObs;
 	}
 
-	private readonly _onDidCreateEditingSession = this._register(new Emitter<IChatEditingSession>());
-	get onDidCreateEditingSession() {
-		return this._onDidCreateEditingSession.event;
-	}
-
 	private readonly _onDidChangeEditingSession = this._register(new Emitter<void>());
 	public readonly onDidChangeEditingSession = this._onDidChangeEditingSession.event;
 
@@ -74,29 +71,34 @@ export class ChatEditingService extends Disposable implements IChatEditingServic
 		return this._editingSessionFileLimit ?? defaultChatEditingMaxFileLimit;
 	}
 
+	private _restoringEditingSession: Promise<IChatEditingSession> | undefined;
+
 	private _applyingChatEditsFailedContextKey: IContextKey<boolean | undefined>;
 
+	private _chatRelatedFilesProviders = new Map<number, IChatRelatedFilesProvider>();
+
 	constructor(
-		@IEditorGroupsService private readonly _editorGroupsService: IEditorGroupsService,
 		@IInstantiationService private readonly _instantiationService: IInstantiationService,
 		@IMultiDiffSourceResolverService multiDiffSourceResolverService: IMultiDiffSourceResolverService,
 		@ITextModelService textModelService: ITextModelService,
 		@IContextKeyService contextKeyService: IContextKeyService,
 		@IChatService private readonly _chatService: IChatService,
 		@IProgressService private readonly _progressService: IProgressService,
-		@ICodeMapperService private readonly _codeMapperService: ICodeMapperService,
 		@IEditorService private readonly _editorService: IEditorService,
 		@IDecorationsService decorationsService: IDecorationsService,
 		@IFileService private readonly _fileService: IFileService,
-		@IWorkbenchAssignmentService private readonly _workbenchAssignmentService: IWorkbenchAssignmentService
+		@ILifecycleService private readonly lifecycleService: ILifecycleService,
+		@IWorkbenchAssignmentService private readonly _workbenchAssignmentService: IWorkbenchAssignmentService,
+		@IStorageService storageService: IStorageService,
+		@ILogService logService: ILogService,
 	) {
 		super();
 		this._applyingChatEditsFailedContextKey = applyingChatEditsFailedContextKey.bindTo(contextKeyService);
 		this._applyingChatEditsFailedContextKey.set(false);
-		this._register(decorationsService.registerDecorationsProvider(new ChatDecorationsProvider(this._currentSessionObs)));
+		this._register(decorationsService.registerDecorationsProvider(_instantiationService.createInstance(ChatDecorationsProvider, this._currentSessionObs)));
 		this._register(multiDiffSourceResolverService.registerResolver(_instantiationService.createInstance(ChatEditingMultiDiffSourceResolver, this._currentSessionObs)));
-		textModelService.registerTextModelContentProvider(ChatEditingTextModelContentProvider.scheme, _instantiationService.createInstance(ChatEditingTextModelContentProvider, this._currentSessionObs));
-		textModelService.registerTextModelContentProvider(ChatEditingSnapshotTextModelContentProvider.scheme, _instantiationService.createInstance(ChatEditingSnapshotTextModelContentProvider, this._currentSessionObs));
+		this._register(textModelService.registerTextModelContentProvider(ChatEditingTextModelContentProvider.scheme, _instantiationService.createInstance(ChatEditingTextModelContentProvider, this._currentSessionObs)));
+		this._register(textModelService.registerTextModelContentProvider(ChatEditingSnapshotTextModelContentProvider.scheme, _instantiationService.createInstance(ChatEditingSnapshotTextModelContentProvider, this._currentSessionObs)));
 		this._register(bindContextKey(decidedChatEditingResourceContextKey, contextKeyService, (reader) => {
 			const currentSession = this._currentSessionObs.read(reader);
 			if (!currentSession) {
@@ -136,38 +138,45 @@ export class ChatEditingService extends Disposable implements IChatEditingServic
 			return this._currentSessionObs.read(r)?.canRedo.read(r) || false;
 		}));
 		this._register(this._chatService.onDidDisposeSession((e) => {
-			this._applyingChatEditsFailedContextKey.set(false);
 			if (e.reason === 'cleared' && this._currentSessionObs.get()?.chatSessionId === e.sessionId) {
+				this._applyingChatEditsFailedContextKey.set(false);
 				void this._currentSessionObs.get()?.stop();
 			}
 		}));
+
+		this._register(this.lifecycleService.onWillShutdown((e) => {
+			const session = this._currentSessionObs.get();
+			if (session) {
+				storageService.store(STORAGE_KEY_EDITING_SESSION, session.chatSessionId, StorageScope.WORKSPACE, StorageTarget.MACHINE);
+				e.join(session.storeState(), { id: 'join.chatEditingSession', label: localize('join.chatEditingSession', "Saving chat edits history") });
+			}
+		}));
+
 		this._editingSessionFileLimitPromise = this._workbenchAssignmentService.getTreatment<number>(chatEditingMaxFileAssignmentName).then(value => {
 			this._editingSessionFileLimit = value ?? defaultChatEditingMaxFileLimit;
 			return this._editingSessionFileLimit;
 		});
 		void this._editingSessionFileLimitPromise;
-	}
 
-	getSnapshotUri(id: string, uri: URI) {
-		const session = this._currentSessionObs.get();
-		if (!session) {
-			return undefined;
-		}
-		return session.getSnapshot(id, uri)?.snapshotUri;
-	}
-
-	getEditingSession(resource: URI): IChatEditingSession | null {
-		const session = this.currentEditingSession;
-		if (!session) {
-			return null;
-		}
-		const entries = session.entries.get();
-		for (const entry of entries) {
-			if (entry.modifiedURI.toString() === resource.toString()) {
-				return session;
+		const sessionIdToRestore = storageService.get(STORAGE_KEY_EDITING_SESSION, StorageScope.WORKSPACE);
+		if (isString(sessionIdToRestore)) {
+			if (this._chatService.getOrRestoreSession(sessionIdToRestore)) {
+				this._restoringEditingSession = this.startOrContinueEditingSession(sessionIdToRestore);
+				this._restoringEditingSession.finally(() => {
+					this._restoringEditingSession = undefined;
+				});
+			} else {
+				logService.error(`Edit session session to restore is a non-existing chat session: ${sessionIdToRestore}`);
 			}
+			storageService.remove(STORAGE_KEY_EDITING_SESSION, StorageScope.WORKSPACE);
 		}
-		return null;
+	}
+
+	async getOrRestoreEditingSession(): Promise<IChatEditingSession | null> {
+		if (this._restoringEditingSession) {
+			await this._restoringEditingSession;
+		}
+		return this.currentEditingSessionObs.get();
 	}
 
 	override dispose(): void {
@@ -175,34 +184,34 @@ export class ChatEditingService extends Disposable implements IChatEditingServic
 		super.dispose();
 	}
 
-	async startOrContinueEditingSession(chatSessionId: string, options?: { silent: boolean }): Promise<IChatEditingSession> {
+	async startOrContinueEditingSession(chatSessionId: string): Promise<IChatEditingSession> {
+		await this._restoringEditingSession;
+
 		const session = this._currentSessionObs.get();
 		if (session) {
-			if (session.chatSessionId !== chatSessionId) {
-				throw new BugIndicatingError('Cannot start new session while another session is active');
+			if (session.chatSessionId === chatSessionId) {
+				return session;
+			} else if (session.chatSessionId !== chatSessionId) {
+				await session.stop(true);
 			}
 		}
-		return this._createEditingSession(chatSessionId, options);
+		return this._createEditingSession(chatSessionId);
 	}
 
-	private async _createEditingSession(chatSessionId: string, options?: { silent: boolean }): Promise<IChatEditingSession> {
+
+	private async _createEditingSession(chatSessionId: string): Promise<IChatEditingSession> {
 		if (this._currentSessionObs.get()) {
 			throw new BugIndicatingError('Cannot have more than one active editing session');
 		}
 
 		this._currentSessionDisposables.clear();
 
+		const session = this._instantiationService.createInstance(ChatEditingSession, chatSessionId, this._editingSessionFileLimitPromise);
+		await session.init();
+
 		// listen for completed responses, run the code mapper and apply the edits to this edit session
-		this._currentSessionDisposables.add(this.installAutoApplyObserver(chatSessionId));
+		this._currentSessionDisposables.add(this.installAutoApplyObserver(session));
 
-		const input = MultiDiffEditorInput.fromResourceMultiDiffEditorInput({
-			multiDiffSource: ChatEditingMultiDiffSourceResolver.getMultiDiffSourceUri(),
-			label: localize('multiDiffEditorInput.name', "Suggested Edits")
-		}, this._instantiationService);
-
-		const editorPane = options?.silent ? undefined : await this._editorGroupsService.activeGroup.openEditor(input, { pinned: true, activation: EditorActivation.ACTIVATE }) as MultiDiffEditor | undefined;
-
-		const session = this._instantiationService.createInstance(ChatEditingSession, chatSessionId, editorPane, this._editingSessionFileLimitPromise);
 		this._currentSessionDisposables.add(session.onDidDispose(() => {
 			this._currentSessionDisposables.clear();
 			this._currentSessionObs.set(null, undefined);
@@ -213,48 +222,29 @@ export class ChatEditingService extends Disposable implements IChatEditingServic
 		}));
 
 		this._currentSessionObs.set(session, undefined);
-		this._onDidCreateEditingSession.fire(session);
 		this._onDidChangeEditingSession.fire();
 		return session;
 	}
 
-	public triggerEditComputation(responseModel: IChatResponseModel): Promise<void> {
-		return this._continueEditingSession(async (builder, token) => {
-			const codeMapperResponse: ICodeMapperResponse = {
-				textEdit: (resource, edits) => builder.textEdits(resource, edits, responseModel),
-			};
-			await this._codeMapperService.mapCodeFromResponse(responseModel, codeMapperResponse, token);
-		}, { silent: true });
-	}
+	private installAutoApplyObserver(session: ChatEditingSession): IDisposable {
 
-	public createSnapshot(requestId: string): void {
-		this._currentSessionObs.get()?.createSnapshot(requestId);
-	}
-
-	public async restoreSnapshot(requestId: string | undefined): Promise<void> {
-		await this._currentSessionObs.get()?.restoreSnapshot(requestId);
-	}
-
-	private installAutoApplyObserver(sessionId: string): IDisposable {
-
-		const chatModel = this._chatService.getSession(sessionId);
+		const chatModel = this._chatService.getOrRestoreSession(session.chatSessionId);
 		if (!chatModel) {
-			throw new Error(`Edit session was created for a non-existing chat session: ${sessionId}`);
+			throw new Error(`Edit session was created for a non-existing chat session: ${session.chatSessionId}`);
 		}
 
 		const observerDisposables = new DisposableStore();
 
 		let editsSource: AsyncIterableSource<IChatTextEditGroup> | undefined;
+		let editsPromise: Promise<void> | undefined;
 		const editsSeen = new ResourceMap<{ seen: number }>();
 		const editedFilesExist = new ResourceMap<Promise<boolean>>();
 
 		const onResponseComplete = (responseModel: IChatResponseModel) => {
-			if (responseModel.result?.errorDetails) {
+			if (responseModel.result?.errorDetails && !responseModel.result.errorDetails.responseIsIncomplete) {
 				// Roll back everything
-				this.restoreSnapshot(responseModel.requestId);
+				session.restoreSnapshot(responseModel.requestId);
 				this._applyingChatEditsFailedContextKey.set(true);
-			} else if (responseModel.result?.metadata?.autoApplyEdits) {
-				this.triggerEditComputation(responseModel);
 			}
 
 			editsSource?.resolve();
@@ -264,7 +254,7 @@ export class ChatEditingService extends Disposable implements IChatEditingServic
 		};
 
 
-		const handleResponseParts = (responseModel: IChatResponseModel) => {
+		const handleResponseParts = async (responseModel: IChatResponseModel) => {
 			for (const part of responseModel.response.value) {
 				if (part.kind === 'codeblockUri' || part.kind === 'textEditGroup') {
 					// ensure editor is open asap
@@ -289,41 +279,55 @@ export class ChatEditingService extends Disposable implements IChatEditingServic
 					const newEdits = allEdits.slice(entry.seen);
 					entry.seen += newEdits.length;
 
-					editsSource ??= new AsyncIterableSource();
-					editsSource.emitOne({ uri: part.uri, edits: newEdits, kind: 'textEditGroup' });
+					if (newEdits.length > 0 || entry.seen === 0) {
+						// only allow empty edits when having just started, ignore otherwise to avoid unneccessary work
+						editsSource ??= new AsyncIterableSource();
+						editsSource.emitOne({ uri: part.uri, edits: newEdits, kind: 'textEditGroup', done: part.kind === 'textEditGroup' && part.done });
+					}
 
 					if (first) {
-						this._continueEditingSession(async (builder, token) => {
+
+						await editsPromise;
+
+						editsPromise = this._continueEditingSession(session, async (builder, token) => {
 							for await (const item of editsSource!.asyncIterable) {
 								if (token.isCancellationRequested) {
 									break;
 								}
-								for (const group of item.edits) {
-									builder.textEdits(item.uri, group, responseModel);
+								if (item.edits.length === 0) {
+									// EMPTY edit, just signal via empty edits that work is starting
+									builder.textEdits(item.uri, [], item.done ?? false, responseModel);
+									continue;
+								}
+								for (let i = 0; i < item.edits.length; i++) {
+									const group = item.edits[i];
+									const isLastGroup = i === item.edits.length - 1;
+									builder.textEdits(item.uri, group, isLastGroup && (item.done ?? false), responseModel);
 								}
 							}
-						}, { silent: true });
+						}).finally(() => {
+							editsPromise = undefined;
+						});
 					}
 				}
 			}
 		};
 
-		observerDisposables.add(chatModel.onDidChange(e => {
+		observerDisposables.add(chatModel.onDidChange(async e => {
 			if (e.kind === 'addRequest') {
+				session.createSnapshot(e.request.id);
 				this._applyingChatEditsFailedContextKey.set(false);
 				const responseModel = e.request.response;
 				if (responseModel) {
 					if (responseModel.isComplete) {
-						handleResponseParts(responseModel);
+						await handleResponseParts(responseModel);
 						onResponseComplete(responseModel);
 					} else {
-						const disposable = responseModel.onDidChange(() => {
-							handleResponseParts(responseModel);
+						const disposable = responseModel.onDidChange(async () => {
+							await handleResponseParts(responseModel);
 							if (responseModel.isComplete) {
 								onResponseComplete(responseModel);
 								disposable.dispose();
-							} else if (responseModel.isCanceled || responseModel.isStale) {
-								disposable.dispose();
 							}
 						});
 					}
@@ -334,48 +338,29 @@ export class ChatEditingService extends Disposable implements IChatEditingServic
 		return observerDisposables;
 	}
 
-	private async _continueEditingSession(builder: (stream: IChatEditingSessionStream, token: CancellationToken) => Promise<void>, options?: { silent?: boolean }): Promise<void> {
-		const session = this._currentSessionObs.get();
-		if (!session) {
-			throw new BugIndicatingError('Cannot continue missing session');
-		}
-
+	private async _continueEditingSession(session: ChatEditingSession, builder: (stream: IChatEditingSessionStream, token: CancellationToken) => Promise<void>): Promise<void> {
 		if (session.state.get() === ChatEditingSessionState.StreamingEdits) {
 			throw new BugIndicatingError('Cannot continue session that is still streaming');
 		}
 
-		let editorPane: MultiDiffEditor | undefined;
-		if (!options?.silent && session.isVisible) {
-			const groupedEditors = this._findGroupedEditors();
-			if (groupedEditors.length !== 1) {
-				throw new Error(`Unexpected number of editors: ${groupedEditors.length}`);
-			}
-			const [group, editor] = groupedEditors[0];
-
-			editorPane = await group.openEditor(editor, { pinned: true, activation: EditorActivation.ACTIVATE }) as MultiDiffEditor | undefined;
-		}
-
 		const stream: IChatEditingSessionStream = {
-			textEdits: (resource: URI, textEdits: TextEdit[], responseModel: IChatResponseModel) => {
-				session.acceptTextEdits(resource, textEdits, responseModel);
+			textEdits: (resource: URI, textEdits: TextEdit[], isDone: boolean, responseModel: IChatResponseModel) => {
+				session.acceptTextEdits(resource, textEdits, isDone, responseModel);
 			}
 		};
 		session.acceptStreamingEditsStart();
 		const cancellationTokenSource = new CancellationTokenSource();
 		this._currentAutoApplyOperationObs.set(cancellationTokenSource, undefined);
 		try {
-			if (editorPane) {
-				await editorPane?.showWhile(builder(stream, cancellationTokenSource.token));
-			} else {
-				await this._progressService.withProgress({
-					location: ProgressLocation.Window,
-					title: localize2('chatEditing.startingSession', 'Generating edits...').value,
-				}, async () => {
-					await builder(stream, cancellationTokenSource.token);
-				},
-					() => cancellationTokenSource.cancel()
-				);
-			}
+			await this._progressService.withProgress({
+				location: ProgressLocation.Window,
+				title: localize2('chatEditing.startingSession', 'Generating edits...').value,
+			}, async () => {
+				await builder(stream, cancellationTokenSource.token);
+			},
+				() => cancellationTokenSource.cancel()
+			);
+
 		} finally {
 			cancellationTokenSource.dispose();
 			this._currentAutoApplyOperationObs.set(null, undefined);
@@ -383,16 +368,45 @@ export class ChatEditingService extends Disposable implements IChatEditingServic
 		}
 	}
 
-	private _findGroupedEditors() {
-		const editors: [IEditorGroup, EditorInput][] = [];
-		for (const group of this._editorGroupsService.groups) {
-			for (const editor of group.editors) {
-				if (editor.resource?.scheme === ChatEditingMultiDiffSourceResolver.scheme) {
-					editors.push([group, editor]);
-				}
+	hasRelatedFilesProviders(): boolean {
+		return this._chatRelatedFilesProviders.size > 0;
+	}
+
+	registerRelatedFilesProvider(handle: number, provider: IChatRelatedFilesProvider): IDisposable {
+		this._chatRelatedFilesProviders.set(handle, provider);
+		return toDisposable(() => {
+			this._chatRelatedFilesProviders.delete(handle);
+		});
+	}
+
+	async getRelatedFiles(chatSessionId: string, prompt: string, token: CancellationToken): Promise<{ group: string; files: IChatRelatedFile[] }[] | undefined> {
+		const currentSession = this._currentSessionObs.get();
+		if (!currentSession || chatSessionId !== currentSession.chatSessionId) {
+			return undefined;
+		}
+		const userAddedWorkingSetEntries: URI[] = [];
+		for (const [uri, metadata] of currentSession.workingSet) {
+			// Don't incorporate suggested files into the related files request
+			// but do consider transient entries like open editors
+			if (metadata.state !== WorkingSetEntryState.Suggested) {
+				userAddedWorkingSetEntries.push(uri);
 			}
 		}
-		return editors;
+
+		const providers = Array.from(this._chatRelatedFilesProviders.values());
+		const result = await Promise.all(providers.map(async provider => {
+			try {
+				const relatedFiles = await provider.provideRelatedFiles({ prompt, files: userAddedWorkingSetEntries }, token);
+				if (relatedFiles?.length) {
+					return { group: provider.description, files: relatedFiles };
+				}
+				return undefined;
+			} catch (e) {
+				return undefined;
+			}
+		}));
+
+		return coalesce(result);
 	}
 }
 
@@ -413,7 +427,7 @@ class ChatDecorationsProvider extends Disposable implements IDecorationsProvider
 
 	readonly label: string = localize('chat', "Chat Editing");
 
-	private readonly _currentlyEditingUris = derived<URI[]>(this, (r) => {
+	private readonly _currentEntries = derived<readonly IModifiedFileEntry[]>(this, (r) => {
 		const session = this._session.read(r);
 		if (!session) {
 			return [];
@@ -422,39 +436,55 @@ class ChatDecorationsProvider extends Disposable implements IDecorationsProvider
 		if (state === ChatEditingSessionState.Disposed) {
 			return [];
 		}
-		return session.entries.read(r).filter(entry => entry.isCurrentlyBeingModified.read(r)).map(entry => entry.modifiedURI);
+		return session.entries.read(r);
 	});
 
-	public readonly onDidChange = observeArrayChanges(this._currentlyEditingUris, compareBy(uri => uri.toString(), compare), this._store);
+	private readonly _currentlyEditingUris = derived<URI[]>(this, (r) => {
+		const uri = this._currentEntries.read(r);
+		return uri.filter(entry => entry.isCurrentlyBeingModified.read(r)).map(entry => entry.modifiedURI);
+	});
+
+	private readonly _modifiedUris = derived<URI[]>(this, (r) => {
+		const uri = this._currentEntries.read(r);
+		return uri.filter(entry => !entry.isCurrentlyBeingModified.read(r) && entry.state.read(r) === WorkingSetEntryState.Modified).map(entry => entry.modifiedURI);
+	});
+
+	public readonly onDidChange = Event.any(
+		observeArrayChanges(this._currentlyEditingUris, compareBy(uri => uri.toString(), compare), this._store),
+		observeArrayChanges(this._modifiedUris, compareBy(uri => uri.toString(), compare), this._store),
+	);
 
 	constructor(
-		private readonly _session: IObservable<IChatEditingSession | null>
+		private readonly _session: IObservable<IChatEditingSession | null>,
+		@IChatAgentService private readonly _chatAgentService: IChatAgentService
 	) {
 		super();
 	}
 
 	provideDecorations(uri: URI, _token: CancellationToken): IDecorationData | undefined {
 		const isCurrentlyBeingModified = this._currentlyEditingUris.get().some(e => e.toString() === uri.toString());
-		if (!isCurrentlyBeingModified) {
-			return undefined;
+		if (isCurrentlyBeingModified) {
+			return {
+				weight: 1000,
+				letter: ThemeIcon.modify(Codicon.loading, 'spin'),
+				bubble: false
+			};
 		}
-		return {
-			weight: 1000,
-			letter: ThemeIcon.modify(Codicon.loading, 'spin'),
-			bubble: false
-		};
+		const isModified = this._modifiedUris.get().some(e => e.toString() === uri.toString());
+		if (isModified) {
+			const defaultAgentName = this._chatAgentService.getDefaultAgent(ChatAgentLocation.EditingSession)?.fullName;
+			return {
+				weight: 1000,
+				letter: Codicon.diffModified,
+				tooltip: defaultAgentName ? localize('chatEditing.modified', "Pending changes from {0}", defaultAgentName) : localize('chatEditing.modified2', "Pending changes from chat"),
+				bubble: true
+			};
+		}
+		return undefined;
 	}
 }
 
 export class ChatEditingMultiDiffSourceResolver implements IMultiDiffSourceResolver {
-	public static readonly scheme = CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME;
-
-	public static getMultiDiffSourceUri(): URI {
-		return URI.from({
-			scheme: ChatEditingMultiDiffSourceResolver.scheme,
-			path: '',
-		});
-	}
 
 	constructor(
 		private readonly _currentSession: IObservable<ChatEditingSession | null>,
@@ -462,7 +492,7 @@ export class ChatEditingMultiDiffSourceResolver implements IMultiDiffSourceResol
 	) { }
 
 	canHandleUri(uri: URI): boolean {
-		return uri.scheme === ChatEditingMultiDiffSourceResolver.scheme;
+		return uri.scheme === CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME;
 	}
 
 	async resolveDiffSource(uri: URI): Promise<IResolvedMultiDiffSource> {
