@@ -10,10 +10,12 @@ import { disposableTimeout, timeout } from '../../../../base/common/async.js';
 import { Codicon } from '../../../../base/common/codicons.js';
 import { toErrorMessage } from '../../../../base/common/errorMessage.js';
 import { Emitter, Event } from '../../../../base/common/event.js';
+import { FuzzyScore } from '../../../../base/common/filters.js';
 import { MarkdownString } from '../../../../base/common/htmlContent.js';
 import { Disposable, DisposableStore, IDisposable, MutableDisposable, combinedDisposable, toDisposable } from '../../../../base/common/lifecycle.js';
 import { ResourceSet } from '../../../../base/common/map.js';
 import { Schemas } from '../../../../base/common/network.js';
+import { autorun } from '../../../../base/common/observable.js';
 import { extUri, isEqual } from '../../../../base/common/resources.js';
 import { isDefined } from '../../../../base/common/types.js';
 import { URI } from '../../../../base/common/uri.js';
@@ -35,7 +37,7 @@ import { asCssVariable } from '../../../../platform/theme/common/colorUtils.js';
 import { IThemeService } from '../../../../platform/theme/common/themeService.js';
 import { ChatAgentLocation, IChatAgentCommand, IChatAgentData, IChatAgentService, IChatWelcomeMessageContent, isChatWelcomeMessageContent } from '../common/chatAgents.js';
 import { ChatContextKeys } from '../common/chatContextKeys.js';
-import { ChatEditingSessionState, IChatEditingService, IChatEditingSession } from '../common/chatEditingService.js';
+import { IChatEditingService, IChatEditingSession, WorkingSetEntryRemovalReason, WorkingSetEntryState } from '../common/chatEditingService.js';
 import { IChatModel, IChatRequestVariableEntry, IChatResponseModel } from '../common/chatModel.js';
 import { ChatRequestAgentPart, IParsedChatRequest, chatAgentLeader, chatSubcommandLeader, formatChatQuestion } from '../common/chatParserTypes.js';
 import { ChatRequestParser } from '../common/chatRequestParser.js';
@@ -47,8 +49,7 @@ import { CodeBlockModelCollection } from '../common/codeBlockModelCollection.js'
 import { ChatTreeItem, IChatAcceptInputOptions, IChatAccessibilityService, IChatCodeBlockInfo, IChatFileTreeInfo, IChatListItemRendererOptions, IChatWidget, IChatWidgetService, IChatWidgetViewContext, IChatWidgetViewOptions } from './chat.js';
 import { ChatAccessibilityProvider } from './chatAccessibilityProvider.js';
 import { ChatAttachmentModel } from './chatAttachmentModel.js';
-import { ChatDragAndDrop } from './chatDragAndDrop.js';
-import { ChatInputPart } from './chatInputPart.js';
+import { ChatInputPart, IChatInputStyles } from './chatInputPart.js';
 import { ChatListDelegate, ChatListItemRenderer, IChatRendererDelegate } from './chatListRenderer.js';
 import { ChatEditorOptions } from './chatOptions.js';
 import './media/chat.css';
@@ -58,21 +59,12 @@ import { ChatViewWelcomePart } from './viewsWelcome/chatViewWelcomeController.js
 
 const $ = dom.$;
 
-function revealLastElement(list: WorkbenchObjectTree<any>) {
-	const newScrollTop = list.scrollHeight - list.renderHeight;
-	list.scrollTop = newScrollTop;
-}
-
 export interface IChatViewState {
 	inputValue?: string;
 	inputState?: IChatInputState;
-	selectedLanguageModelId?: string;
 }
 
-export interface IChatWidgetStyles {
-	listForeground: string;
-	listBackground: string;
-	overlayBackground: string;
+export interface IChatWidgetStyles extends IChatInputStyles {
 	inputEditorBackground: string;
 	resultEditorBackground: string;
 }
@@ -143,9 +135,10 @@ export class ChatWidget extends Disposable implements IChatWidget {
 
 	private contribs: ReadonlyArray<IChatWidgetContrib> = [];
 
-	private tree!: WorkbenchObjectTree<ChatTreeItem>;
+	private tree!: WorkbenchObjectTree<ChatTreeItem, FuzzyScore>;
 	private renderer!: ChatListItemRenderer;
 	private readonly _codeBlockModelCollection: CodeBlockModelCollection;
+	private lastItem: ChatTreeItem | undefined;
 
 	private inputPart!: ChatInputPart;
 	private editorOptions!: ChatEditorOptions;
@@ -258,7 +251,11 @@ export class ChatWidget extends Disposable implements IChatWidget {
 
 		const chatEditingSessionDisposables = this._register(new DisposableStore());
 
-		this._register(this.chatEditingService.onDidCreateEditingSession((session) => {
+		this._register(autorun(r => {
+			const session = this.chatEditingService.currentEditingSessionObs.read(r);
+			if (!session) {
+				return;
+			}
 			if (session.chatSessionId !== this.viewModel?.sessionId) {
 				// this chat editing session is for a different chat widget
 				return;
@@ -289,16 +286,16 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		if (this._location.location === ChatAgentLocation.EditingSession) {
 			let currentEditSession: IChatEditingSession | undefined = undefined;
 			this._register(this.onDidChangeViewModel(async () => {
-
 				const sessionId = this._viewModel?.sessionId;
-				if (sessionId !== currentEditSession?.chatSessionId) {
-					if (currentEditSession && (currentEditSession.state.get() !== ChatEditingSessionState.Disposed)) {
-						await currentEditSession.stop();
+				if (sessionId) {
+					if (sessionId !== currentEditSession?.chatSessionId) {
+						currentEditSession = await this.chatEditingService.startOrContinueEditingSession(sessionId);
 					}
-					if (sessionId) {
-						currentEditSession = await this.chatEditingService.startOrContinueEditingSession(sessionId, { silent: true });
-					} else {
+				} else {
+					if (currentEditSession) {
+						const session = currentEditSession;
 						currentEditSession = undefined;
+						await session.stop();
 					}
 				}
 			}));
@@ -434,7 +431,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		scrollDownButton.setTitle(localize('scrollDownButtonLabel', "Scroll down"));
 		this._register(scrollDownButton.onDidClick(() => {
 			this.scrollLock = true;
-			revealLastElement(this.tree);
+			this.scrollToEnd();
 		}));
 
 		this._register(this.editorOptions.onDidChange(() => this.onDidStyleChange()));
@@ -443,7 +440,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		// Do initial render
 		if (this.viewModel) {
 			this.onDidChangeItems();
-			revealLastElement(this.tree);
+			this.scrollToEnd();
 		}
 
 		this.contribs = ChatWidget.CONTRIBS.map(contrib => {
@@ -455,10 +452,16 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			}
 		}).filter(isDefined);
 
-		this._register(this.instantiationService.createInstance(ChatDragAndDrop, this.container, this.inputPart, this.styles));
 		this._register((this.chatWidgetService as ChatWidgetService).register(this));
 	}
 
+	private scrollToEnd() {
+		if (this.lastItem) {
+			const offset = Math.max(this.lastItem.currentRenderedHeight ?? 0, 1e6);
+			this.tree.reveal(this.lastItem, offset);
+		}
+	}
+
 	getContrib<T extends IChatWidgetContrib>(id: string): T | undefined {
 		return this.contribs.find(c => c.id === id) as T;
 	}
@@ -507,7 +510,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	}
 
 	private onDidChangeItems(skipDynamicLayout?: boolean) {
-		if (this.tree && this._visible) {
+		if (this._visible || !this.viewModel) {
 			const treeItems = (this.viewModel?.getItems() ?? [])
 				.map((item): ITreeElement<ChatTreeItem> => {
 					return {
@@ -521,6 +524,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 
 			this._onWillMaybeChangeHeight.fire();
 
+			this.lastItem = treeItems.at(-1)?.element;
+			ChatContextKeys.lastItemId.bindTo(this.contextKeyService).set(this.lastItem ? [this.lastItem.id] : []);
 			this.tree.setChildren(null, treeItems, {
 				diffIdentityProvider: {
 					getId: (element) => {
@@ -533,7 +538,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 							// Re-render once content references are loaded
 							(isResponseVM(element) ? `_${element.contentReferences.length}` : '') +
 							// Re-render if element becomes hidden due to undo/redo
-							`_${element.isHidden ? '1' : '0'}` +
+							`_${element.shouldBeRemovedOnSend ? '1' : '0'}` +
 							// Rerender request if we got new content references in the response
 							// since this may change how we render the corresponding attachments in the request
 							(isRequestVM(element) && element.contentReferences ? `_${element.contentReferences?.length}` : '');
@@ -545,12 +550,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				this.layoutDynamicChatTreeItemMode();
 			}
 
-			const lastItem = treeItems[treeItems.length - 1]?.element;
-			if (lastItem) {
-				ChatContextKeys.lastItemId.bindTo(this.contextKeyService).set([lastItem.id]);
-			}
-			if (lastItem && isResponseVM(lastItem) && lastItem.isComplete) {
-				this.renderFollowups(lastItem.replyFollowups, lastItem);
+			if (this.lastItem && isResponseVM(this.lastItem) && this.lastItem.isComplete) {
+				this.renderFollowups(this.lastItem.replyFollowups, this.lastItem);
 			} else if (!treeItems.length && this.viewModel) {
 				this.renderFollowups(this.viewModel.model.sampleQuestions);
 			} else {
@@ -655,8 +656,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			}
 		}));
 
-		this.tree = this._register(<WorkbenchObjectTree<ChatTreeItem>>scopedInstantiationService.createInstance(
-			WorkbenchObjectTree,
+		this.tree = this._register(scopedInstantiationService.createInstance(
+			WorkbenchObjectTree<ChatTreeItem, FuzzyScore>,
 			'Chat',
 			listContainer,
 			delegate,
@@ -738,10 +739,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 					dom.scheduleAtNextAnimationFrame(dom.getWindow(this.listContainer), () => {
 						// Can't set scrollTop during this event listener, the list might overwrite the change
 
-						// TODO This doesn't necessarily work on the first try because of the dynamic heights of items and
-						// the way ListView works. Twice is usually enough. But this would work better if this lived inside ListView somehow
-						revealLastElement(this.tree);
-						revealLastElement(this.tree);
+						this.scrollToEnd();
 					}, 0);
 				}
 			}
@@ -765,6 +763,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				editorOverflowWidgetsDomNode: this.viewOptions.editorOverflowWidgetsDomNode,
 				enableImplicitContext: this.viewOptions.enableImplicitContext
 			},
+			this.styles,
 			() => this.collectInputState()
 		));
 		this.inputPart.render(container, '', this);
@@ -868,7 +867,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 
 			this.onDidChangeItems();
 			if (events.some(e => e?.kind === 'addRequest') && this.visible) {
-				revealLastElement(this.tree); // Now we know how big they actually are... how do we know that 2 rounds is enough
+				this.scrollToEnd();
 			}
 
 			if (this.chatEditingService.currentEditingSession && this.chatEditingService.currentEditingSession?.chatSessionId === this.viewModel?.sessionId) {
@@ -894,16 +893,11 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			if (e.kind === 'setAgent') {
 				this._onDidChangeAgent.fire({ agent: e.agent, slashCommand: e.command });
 			}
-			if (e.kind === 'addRequest') {
-				if (this._location.location === ChatAgentLocation.EditingSession) {
-					this.chatEditingService.createSnapshot(e.request.id);
-				}
-			}
 		}));
 
 		if (this.tree && this.visible) {
 			this.onDidChangeItems();
-			revealLastElement(this.tree);
+			this.scrollToEnd();
 		}
 
 		this.updateChatInputContext();
@@ -957,6 +951,25 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		return this._acceptInput(query ? { query } : undefined, options);
 	}
 
+	async rerunLastRequest(): Promise<void> {
+		if (!this.viewModel) {
+			return;
+		}
+
+		const sessionId = this.viewModel.sessionId;
+		const lastRequest = this.chatService.getSession(sessionId)?.getRequests().at(-1);
+		if (!lastRequest) {
+			return;
+		}
+
+		const options: IChatSendRequestOptions = {
+			attempt: lastRequest.attempt + 1,
+			location: this.location,
+			userSelectedModelId: this.input.currentLanguageModel
+		};
+		return await this.chatService.resendRequest(lastRequest, options);
+	}
+
 	async acceptInputWithPrefix(prefix: string): Promise<void> {
 		this._acceptInput({ prefix });
 	}
@@ -972,6 +985,10 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	}
 
 	private async _acceptInput(query: { query: string } | { prefix: string } | undefined, options?: IChatAcceptInputOptions): Promise<IChatResponseModel | undefined> {
+		if (this.viewModel?.requestInProgress) {
+			return;
+		}
+
 		if (this.viewModel) {
 			this._onDidAcceptInput.fire();
 			if (!this.viewOptions.autoScroll) {
@@ -985,24 +1002,27 @@ export class ChatWidget extends Disposable implements IChatWidget {
 					`${query.prefix} ${editorValue}`;
 			const isUserQuery = !query || 'prefix' in query;
 
-			const requests = this.viewModel.model.getRequests();
-			for (let i = requests.length - 1; i >= 0; i -= 1) {
-				const request = requests[i];
-				if (request.isHidden) {
-					this.chatService.removeRequest(this.viewModel.sessionId, request.id);
-				}
-			}
 
-			let attachedContext = this.inputPart.getAttachedAndImplicitContext();
+
+			let attachedContext = this.inputPart.getAttachedAndImplicitContext(this.viewModel.sessionId);
 			let workingSet: URI[] | undefined;
 			if (this.location === ChatAgentLocation.EditingSession) {
+				const currentEditingSession = this.chatEditingService.currentEditingSessionObs.get();
+
+				const unconfirmedSuggestions = new ResourceSet();
 				const uniqueWorkingSetEntries = new ResourceSet(); // NOTE: this is used for bookkeeping so the UI can avoid rendering references in the UI that are already shown in the working set
-				const editingSessionAttachedContext: IChatRequestVariableEntry[] = this.inputPart.chatEditWorkingSetFiles.map((v) => {
-					// Pick up everything that the user sees is part of the working set.
-					// This should never exceed the maximum file entries limit above.
-					uniqueWorkingSetEntries.add(v);
-					return this.attachmentModel.asVariableEntry(v);
-				});
+				const editingSessionAttachedContext: IChatRequestVariableEntry[] = [];
+				// Pick up everything that the user sees is part of the working set.
+				// This should never exceed the maximum file entries limit above.
+				for (const v of this.inputPart.chatEditWorkingSetFiles) {
+					// Skip over any suggested files that haven't been confirmed yet in the working set
+					if (currentEditingSession?.workingSet.get(v)?.state === WorkingSetEntryState.Suggested) {
+						unconfirmedSuggestions.add(v);
+					} else {
+						uniqueWorkingSetEntries.add(v);
+						editingSessionAttachedContext.push(this.attachmentModel.asVariableEntry(v));
+					}
+				}
 				let maximumFileEntries = this.chatEditingService.editingSessionFileLimit - editingSessionAttachedContext.length;
 
 				// Then take any attachments that are not files
@@ -1012,7 +1032,6 @@ export class ChatWidget extends Disposable implements IChatWidget {
 					}
 				}
 
-				const currentEditingSession = this.chatEditingService.currentEditingSessionObs.get();
 				for (const file of uniqueWorkingSetEntries) {
 					// Make sure that any files that we sent are part of the working set
 					// but do not permanently add file variables from previous requests to the working set
@@ -1048,6 +1067,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 					actualSize: number;
 				};
 				this.telemetryService.publicLog2<ChatEditingWorkingSetEvent, ChatEditingWorkingSetClassification>('chatEditing/workingSetSize', { originalSize: this.inputPart.attemptedWorkingSetEntriesCount, actualSize: uniqueWorkingSetEntries.size });
+				currentEditingSession?.remove(WorkingSetEntryRemovalReason.User, ...unconfirmedSuggestions);
 			}
 
 			const result = await this.chatService.sendRequest(this.viewModel.sessionId, input, {
@@ -1138,7 +1158,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		const lastItem = this.viewModel?.getItems().at(-1);
 		const lastResponseIsRendering = isResponseVM(lastItem) && lastItem.renderData;
 		if (lastElementVisible && (!lastResponseIsRendering || this.viewOptions.autoScroll)) {
-			revealLastElement(this.tree);
+			this.scrollToEnd();
 		}
 
 		this.listContainer.style.height = `${listHeight}px`;
@@ -1241,8 +1261,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		);
 
 		if (needsRerender || !listHeight) {
-			// TODO: figure out a better place to reveal the last element
-			revealLastElement(this.tree);
+			this.scrollToEnd();
 		}
 	}
 
@@ -1257,8 +1276,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	getViewState(): IChatViewState {
 		return {
 			inputValue: this.getInput(),
-			inputState: this.inputPart.getViewState(),
-			selectedLanguageModelId: this.inputPart.currentLanguageModel,
+			inputState: this.inputPart.getViewState()
 		};
 	}
 
@@ -1282,7 +1300,7 @@ export class ChatWidgetService extends Disposable implements IChatWidgetService
 		return this._lastFocusedWidget;
 	}
 
-	getAllWidgets(location: ChatAgentLocation): ReadonlyArray<IChatWidget> {
+	getWidgetsByLocations(location: ChatAgentLocation): ReadonlyArray<IChatWidget> {
 		return this._widgets.filter(w => w.location === location);
 	}
 
@@ -1290,10 +1308,6 @@ export class ChatWidgetService extends Disposable implements IChatWidgetService
 		return this._widgets.find(w => isEqual(w.inputUri, uri));
 	}
 
-	getWidgetByLocation(location: ChatAgentLocation): ChatWidget[] {
-		return this._widgets.filter(w => w.location === location);
-	}
-
 	getWidgetBySessionId(sessionId: string): ChatWidget | undefined {
 		return this._widgets.find(w => w.viewModel?.sessionId === sessionId);
 	}
