@@ -275,9 +275,6 @@ export class ExtensionEditor extends EditorPane {
 		@IViewsService private readonly viewsService: IViewsService,
 		@IUriIdentityService private readonly uriIdentityService: IUriIdentityService,
 		@IHoverService private readonly hoverService: IHoverService,
-		@IUserDataProfilesService private readonly userDataProfilesService: IUserDataProfilesService,
-		@IRemoteAgentService private readonly remoteAgentService: IRemoteAgentService,
-		@IFileService private readonly fileService: IFileService,
 	) {
 		super(ExtensionEditor.ID, group, telemetryService, themeService, storageService);
 		this.extensionReadme = null;
@@ -946,186 +943,12 @@ export class ExtensionEditor extends EditorPane {
 		this.contentDisposables.add(toDisposable(removeLayoutParticipant));
 		this.contentDisposables.add(scrollableContent);
 
-		if (extension.local) {
-			this.renderInstallInfo(content, extension.local);
-		}
-		if (extension.gallery) {
-			this.renderMarketplaceInfo(content, extension);
-		}
-		this.renderCategories(content, extension);
-		this.renderExtensionResources(content, extension);
+		this.contentDisposables.add(this.instantiationService.createInstance(AdditionalDetailsWidget, content, extension));
 
 		append(container, scrollableContent.getDomNode());
 		scrollableContent.scanDomNode();
 	}
 
-	private renderCategories(container: HTMLElement, extension: IExtension): void {
-		if (extension.categories.length) {
-			const categoriesContainer = append(container, $('.categories-container.additional-details-element'));
-			append(categoriesContainer, $('.additional-details-title', undefined, localize('categories', "Categories")));
-			const categoriesElement = append(categoriesContainer, $('.categories'));
-			for (const category of extension.categories) {
-				this.transientDisposables.add(onClick(append(categoriesElement, $('span.category', { tabindex: '0' }, category)),
-					() => this.extensionsWorkbenchService.openSearch(`@category:"${category}"`)));
-			}
-		}
-	}
-
-	private renderExtensionResources(container: HTMLElement, extension: IExtension): void {
-		const resources: [string, URI][] = [];
-		if (extension.url) {
-			resources.push([localize('Marketplace', "Marketplace"), URI.parse(extension.url)]);
-		}
-		if (extension.url && extension.supportUrl) {
-			try {
-				resources.push([localize('issues', "Issues"), URI.parse(extension.supportUrl)]);
-			} catch (error) {/* Ignore */ }
-		}
-		if (extension.repository) {
-			try {
-				resources.push([localize('repository', "Repository"), URI.parse(extension.repository)]);
-			} catch (error) {/* Ignore */ }
-		}
-		if (extension.url && extension.licenseUrl) {
-			try {
-				resources.push([localize('license', "License"), URI.parse(extension.licenseUrl)]);
-			} catch (error) {/* Ignore */ }
-		}
-		if (extension.publisherUrl) {
-			resources.push([extension.publisherDisplayName, extension.publisherUrl]);
-		}
-		if (resources.length || extension.publisherSponsorLink) {
-			const extensionResourcesContainer = append(container, $('.resources-container.additional-details-element'));
-			append(extensionResourcesContainer, $('.additional-details-title', undefined, localize('resources', "Resources")));
-			const resourcesElement = append(extensionResourcesContainer, $('.resources'));
-			for (const [label, uri] of resources) {
-				const resource = append(resourcesElement, $('a.resource', { tabindex: '0' }, label));
-				this.transientDisposables.add(onClick(resource, () => {
-					this.openerService.open(uri);
-				}));
-				this.transientDisposables.add(this.hoverService.setupManagedHover(getDefaultHoverDelegate('mouse'), resource, uri.toString()));
-			}
-		}
-	}
-
-	private renderInstallInfo(container: HTMLElement, extension: ILocalExtension): void {
-		const installInfoContainer = append(container, $('.more-info-container.additional-details-element'));
-		append(installInfoContainer, $('.additional-details-title', undefined, localize('Install Info', "Installation")));
-		const installInfo = append(installInfoContainer, $('.more-info'));
-		append(installInfo,
-			$('.more-info-entry', undefined,
-				$('div', undefined, localize('id', "Identifier")),
-				$('code', undefined, extension.identifier.id)
-			));
-		append(installInfo,
-			$('.more-info-entry', undefined,
-				$('div', undefined, localize('Version', "Version")),
-				$('code', undefined, extension.manifest.version)
-			)
-		);
-		if (extension.installedTimestamp) {
-			append(installInfo,
-				$('.more-info-entry', undefined,
-					$('div', undefined, localize('last updated', "Last Updated")),
-					$('div', undefined, toDateString(new Date(extension.installedTimestamp)))
-				)
-			);
-		}
-		if (extension.source !== 'gallery') {
-			const element = $('div', undefined, extension.source === 'vsix' ? localize('vsix', "VSIX") : localize('other', "Local"));
-			append(installInfo,
-				$('.more-info-entry', undefined,
-					$('div', undefined, localize('source', "Source")),
-					element
-				)
-			);
-			if (isNative && extension.source === 'resource' && extension.location.scheme === Schemas.file) {
-				element.classList.add('link');
-				element.title = extension.location.fsPath;
-				this.transientDisposables.add(onClick(element, () => this.openerService.open(extension.location, { openExternal: true })));
-			}
-		}
-		if (extension.size) {
-			const element = $('div', undefined, toMemoryString(extension.size));
-			append(installInfo,
-				$('.more-info-entry', undefined,
-					$('div', { title: localize('size when installed', "Size when installed") }, localize('size', "Size")),
-					element
-				)
-			);
-			if (isNative && extension.location.scheme === Schemas.file) {
-				element.classList.add('link');
-				element.title = extension.location.fsPath;
-				this.transientDisposables.add(onClick(element, () => this.openerService.open(extension.location, { openExternal: true })));
-			}
-		}
-		this.getCacheLocation(extension).then(cacheLocation => {
-			if (!cacheLocation) {
-				return;
-			}
-			computeSize(cacheLocation, this.fileService).then(cacheSize => {
-				if (!cacheSize) {
-					return;
-				}
-				const element = $('div', undefined, toMemoryString(cacheSize));
-				append(installInfo,
-					$('.more-info-entry', undefined,
-						$('div', { title: localize('disk space used', "Cache size") }, localize('cache size', "Cache")),
-						element)
-				);
-				if (isNative && extension.location.scheme === Schemas.file) {
-					element.classList.add('link');
-					element.title = cacheLocation.fsPath;
-					this.transientDisposables.add(onClick(element, () => this.openerService.open(cacheLocation.with({ scheme: Schemas.file }), { openExternal: true })));
-				}
-			});
-		});
-	}
-
-	private async getCacheLocation(extension: ILocalExtension): Promise<URI | undefined> {
-		let extensionCacheLocation = this.uriIdentityService.extUri.joinPath(this.userDataProfilesService.defaultProfile.globalStorageHome, extension.identifier.id.toLowerCase());
-		if (extension.location.scheme === Schemas.vscodeRemote) {
-			const environment = await this.remoteAgentService.getEnvironment();
-			if (!environment) {
-				return undefined;
-			}
-			extensionCacheLocation = this.uriIdentityService.extUri.joinPath(environment.globalStorageHome, extension.identifier.id.toLowerCase());
-		}
-		return extensionCacheLocation;
-	}
-
-	private renderMarketplaceInfo(container: HTMLElement, extension: IExtension): void {
-		const gallery = extension.gallery;
-		const moreInfoContainer = append(container, $('.more-info-container.additional-details-element'));
-		append(moreInfoContainer, $('.additional-details-title', undefined, localize('Marketplace Info', "Marketplace")));
-		const moreInfo = append(moreInfoContainer, $('.more-info'));
-		if (gallery) {
-			if (!extension.local) {
-				append(moreInfo,
-					$('.more-info-entry', undefined,
-						$('div', undefined, localize('id', "Identifier")),
-						$('code', undefined, extension.identifier.id)
-					));
-				append(moreInfo,
-					$('.more-info-entry', undefined,
-						$('div', undefined, localize('Version', "Version")),
-						$('code', undefined, gallery.version)
-					)
-				);
-			}
-			append(moreInfo,
-				$('.more-info-entry', undefined,
-					$('div', undefined, localize('published', "Published")),
-					$('div', undefined, toDateString(new Date(gallery.releaseDate)))
-				),
-				$('.more-info-entry', undefined,
-					$('div', undefined, localize('last released', "Last Released")),
-					$('div', undefined, toDateString(new Date(gallery.lastUpdated)))
-				)
-			);
-		}
-	}
-
 	private openChangelog(extension: IExtension, template: IExtensionEditorTemplate, token: CancellationToken): Promise<IActiveElement | null> {
 		return this.openMarkdown(extension, this.extensionChangelog!.get(), localize('noChangelog', "No Changelog available."), template.content, WebviewIndex.Changelog, localize('Changelog title', "Changelog"), token);
 	}
@@ -1240,6 +1063,212 @@ export class ExtensionEditor extends EditorPane {
 	}
 }
 
+class AdditionalDetailsWidget extends Disposable {
+
+	private readonly disposables = this._register(new DisposableStore());
+
+	constructor(
+		private readonly container: HTMLElement,
+		extension: IExtension,
+		@IHoverService private readonly hoverService: IHoverService,
+		@IOpenerService private readonly openerService: IOpenerService,
+		@IUserDataProfilesService private readonly userDataProfilesService: IUserDataProfilesService,
+		@IRemoteAgentService private readonly remoteAgentService: IRemoteAgentService,
+		@IFileService private readonly fileService: IFileService,
+		@IUriIdentityService private readonly uriIdentityService: IUriIdentityService,
+		@IExtensionsWorkbenchService private readonly extensionsWorkbenchService: IExtensionsWorkbenchService,
+	) {
+		super();
+		this.render(extension);
+		this._register(this.extensionsWorkbenchService.onChange(e => {
+			if (e && areSameExtensions(e.identifier, extension.identifier) && e.server === extension.server) {
+				this.render(e);
+			}
+		}));
+	}
+
+	private render(extension: IExtension): void {
+		this.container.innerText = '';
+		this.disposables.clear();
+
+		if (extension.local) {
+			this.renderInstallInfo(this.container, extension.local);
+		}
+		if (extension.gallery) {
+			this.renderMarketplaceInfo(this.container, extension);
+		}
+		this.renderCategories(this.container, extension);
+		this.renderExtensionResources(this.container, extension);
+	}
+
+	private renderCategories(container: HTMLElement, extension: IExtension): void {
+		if (extension.categories.length) {
+			const categoriesContainer = append(container, $('.categories-container.additional-details-element'));
+			append(categoriesContainer, $('.additional-details-title', undefined, localize('categories', "Categories")));
+			const categoriesElement = append(categoriesContainer, $('.categories'));
+			for (const category of extension.categories) {
+				this.disposables.add(onClick(append(categoriesElement, $('span.category', { tabindex: '0' }, category)),
+					() => this.extensionsWorkbenchService.openSearch(`@category:"${category}"`)));
+			}
+		}
+	}
+
+	private renderExtensionResources(container: HTMLElement, extension: IExtension): void {
+		const resources: [string, URI][] = [];
+		if (extension.url) {
+			resources.push([localize('Marketplace', "Marketplace"), URI.parse(extension.url)]);
+		}
+		if (extension.url && extension.supportUrl) {
+			try {
+				resources.push([localize('issues', "Issues"), URI.parse(extension.supportUrl)]);
+			} catch (error) {/* Ignore */ }
+		}
+		if (extension.repository) {
+			try {
+				resources.push([localize('repository', "Repository"), URI.parse(extension.repository)]);
+			} catch (error) {/* Ignore */ }
+		}
+		if (extension.url && extension.licenseUrl) {
+			try {
+				resources.push([localize('license', "License"), URI.parse(extension.licenseUrl)]);
+			} catch (error) {/* Ignore */ }
+		}
+		if (extension.publisherUrl) {
+			resources.push([extension.publisherDisplayName, extension.publisherUrl]);
+		}
+		if (resources.length || extension.publisherSponsorLink) {
+			const extensionResourcesContainer = append(container, $('.resources-container.additional-details-element'));
+			append(extensionResourcesContainer, $('.additional-details-title', undefined, localize('resources', "Resources")));
+			const resourcesElement = append(extensionResourcesContainer, $('.resources'));
+			for (const [label, uri] of resources) {
+				const resource = append(resourcesElement, $('a.resource', { tabindex: '0' }, label));
+				this.disposables.add(onClick(resource, () => this.openerService.open(uri)));
+				this.disposables.add(this.hoverService.setupManagedHover(getDefaultHoverDelegate('mouse'), resource, uri.toString()));
+			}
+		}
+	}
+
+	private renderInstallInfo(container: HTMLElement, extension: ILocalExtension): void {
+		const installInfoContainer = append(container, $('.more-info-container.additional-details-element'));
+		append(installInfoContainer, $('.additional-details-title', undefined, localize('Install Info', "Installation")));
+		const installInfo = append(installInfoContainer, $('.more-info'));
+		append(installInfo,
+			$('.more-info-entry', undefined,
+				$('div.more-info-entry-name', undefined, localize('id', "Identifier")),
+				$('code', undefined, extension.identifier.id)
+			));
+		if (extension.type !== ExtensionType.System) {
+			append(installInfo,
+				$('.more-info-entry', undefined,
+					$('div.more-info-entry-name', undefined, localize('Version', "Version")),
+					$('code', undefined, extension.manifest.version)
+				)
+			);
+		}
+		if (extension.installedTimestamp) {
+			append(installInfo,
+				$('.more-info-entry', undefined,
+					$('div.more-info-entry-name', undefined, localize('last updated', "Last Updated")),
+					$('div', undefined, toDateString(new Date(extension.installedTimestamp)))
+				)
+			);
+		}
+		if (!extension.isBuiltin && extension.source !== 'gallery') {
+			const element = $('div', undefined, extension.source === 'vsix' ? localize('vsix', "VSIX") : localize('other', "Local"));
+			append(installInfo,
+				$('.more-info-entry', undefined,
+					$('div.more-info-entry-name', undefined, localize('source', "Source")),
+					element
+				)
+			);
+			if (isNative && extension.source === 'resource' && extension.location.scheme === Schemas.file) {
+				element.classList.add('link');
+				element.title = extension.location.fsPath;
+				this.disposables.add(onClick(element, () => this.openerService.open(extension.location, { openExternal: true })));
+			}
+		}
+		if (extension.size) {
+			const element = $('div', undefined, toMemoryString(extension.size));
+			append(installInfo,
+				$('.more-info-entry', undefined,
+					$('div.more-info-entry-name', { title: localize('size when installed', "Size when installed") }, localize('size', "Size")),
+					element
+				)
+			);
+			if (isNative && extension.location.scheme === Schemas.file) {
+				element.classList.add('link');
+				element.title = extension.location.fsPath;
+				this.disposables.add(onClick(element, () => this.openerService.open(extension.location, { openExternal: true })));
+			}
+		}
+		this.getCacheLocation(extension).then(cacheLocation => {
+			if (!cacheLocation) {
+				return;
+			}
+			computeSize(cacheLocation, this.fileService).then(cacheSize => {
+				if (!cacheSize) {
+					return;
+				}
+				const element = $('div', undefined, toMemoryString(cacheSize));
+				append(installInfo,
+					$('.more-info-entry', undefined,
+						$('div.more-info-entry-name', { title: localize('disk space used', "Cache size") }, localize('cache size', "Cache")),
+						element)
+				);
+				if (isNative && extension.location.scheme === Schemas.file) {
+					element.classList.add('link');
+					element.title = cacheLocation.fsPath;
+					this.disposables.add(onClick(element, () => this.openerService.open(cacheLocation.with({ scheme: Schemas.file }), { openExternal: true })));
+				}
+			});
+		});
+	}
+
+	private async getCacheLocation(extension: ILocalExtension): Promise<URI | undefined> {
+		let extensionCacheLocation = this.uriIdentityService.extUri.joinPath(this.userDataProfilesService.defaultProfile.globalStorageHome, extension.identifier.id.toLowerCase());
+		if (extension.location.scheme === Schemas.vscodeRemote) {
+			const environment = await this.remoteAgentService.getEnvironment();
+			if (!environment) {
+				return undefined;
+			}
+			extensionCacheLocation = this.uriIdentityService.extUri.joinPath(environment.globalStorageHome, extension.identifier.id.toLowerCase());
+		}
+		return extensionCacheLocation;
+	}
+
+	private renderMarketplaceInfo(container: HTMLElement, extension: IExtension): void {
+		const gallery = extension.gallery;
+		const moreInfoContainer = append(container, $('.more-info-container.additional-details-element'));
+		append(moreInfoContainer, $('.additional-details-title', undefined, localize('Marketplace Info', "Marketplace")));
+		const moreInfo = append(moreInfoContainer, $('.more-info'));
+		if (gallery) {
+			if (!extension.local) {
+				append(moreInfo,
+					$('.more-info-entry', undefined,
+						$('div.more-info-entry-name', undefined, localize('id', "Identifier")),
+						$('code', undefined, extension.identifier.id)
+					));
+				append(moreInfo,
+					$('.more-info-entry', undefined,
+						$('div.more-info-entry-name', undefined, localize('Version', "Version")),
+						$('code', undefined, gallery.version)
+					)
+				);
+			}
+			append(moreInfo,
+				$('.more-info-entry', undefined,
+					$('div.more-info-entry-name', undefined, localize('published', "Published")),
+					$('div', undefined, toDateString(new Date(gallery.releaseDate)))
+				),
+				$('.more-info-entry', undefined,
+					$('div.more-info-entry-name', undefined, localize('last released', "Last Released")),
+					$('div', undefined, toDateString(new Date(gallery.lastUpdated)))
+				)
+			);
+		}
+	}
+}
+
 const contextKeyExpr = ContextKeyExpr.and(ContextKeyExpr.equals('activeEditor', ExtensionEditor.ID), EditorContextKeys.focus.toNegated());
 registerAction2(class ShowExtensionEditorFindAction extends Action2 {
 	constructor() {
