@@ -44,7 +44,7 @@ import { Action2, IMenu, IMenuService, MenuId, MenuItemAction, MenuRegistry, reg
 import { ContextKeyExpr, IContextKeyService, RawContextKey } from '../../../../../../platform/contextkey/common/contextkey.js';
 import { MenuEntryActionViewItem, getActionBarActions } from '../../../../../../platform/actions/browser/menuEntryActionViewItem.js';
 import { IAction } from '../../../../../../base/common/actions.js';
-import { NotebookSectionArgs } from '../../controller/sectionActions.js';
+import { NotebookOutlineEntryArgs } from '../../controller/sectionActions.js';
 import { MarkupCellViewModel } from '../../viewModel/markupCellViewModel.js';
 import { Delayer, disposableTimeout } from '../../../../../../base/common/async.js';
 import { IOutlinePane } from '../../../../outline/browser/outline.js';
@@ -153,8 +153,12 @@ class NotebookOutlineRenderer implements ITreeRenderer<OutlineEntry, FuzzyScore,
 		}
 
 		if (this._target === OutlineTarget.OutlinePane) {
+			if (!this._editor) {
+				return;
+			}
+
 			const nbCell = node.element.cell;
-			const nbViewModel = this._editor?.getViewModel();
+			const nbViewModel = this._editor.getViewModel();
 			if (!nbViewModel) {
 				return;
 			}
@@ -181,7 +185,7 @@ class NotebookOutlineRenderer implements ITreeRenderer<OutlineEntry, FuzzyScore,
 			const actions = getOutlineToolbarActions(menu, { notebookEditor: this._editor, outlineEntry: node.element });
 			outlineEntryToolbar.setActions(actions.primary, actions.secondary);
 
-			this.setupToolbarListeners(outlineEntryToolbar, menu, actions, node.element, template);
+			this.setupToolbarListeners(this._editor, outlineEntryToolbar, menu, actions, node.element, template);
 			template.actionMenu.style.padding = '0 0.8em 0 0.4em';
 		}
 	}
@@ -210,7 +214,7 @@ class NotebookOutlineRenderer implements ITreeRenderer<OutlineEntry, FuzzyScore,
 		}
 	}
 
-	private setupToolbarListeners(toolbar: ToolBar, menu: IMenu, initActions: { primary: IAction[]; secondary: IAction[] }, entry: OutlineEntry, templateData: NotebookOutlineTemplate): void {
+	private setupToolbarListeners(editor: INotebookEditor, toolbar: ToolBar, menu: IMenu, initActions: { primary: IAction[]; secondary: IAction[] }, entry: OutlineEntry, templateData: NotebookOutlineTemplate): void {
 		// same fix as in cellToolbars setupListeners re #103926
 		let dropdownIsVisible = false;
 		let deferredUpdate: (() => void) | undefined;
@@ -218,13 +222,13 @@ class NotebookOutlineRenderer implements ITreeRenderer<OutlineEntry, FuzzyScore,
 		toolbar.setActions(initActions.primary, initActions.secondary);
 		templateData.elementDisposables.add(menu.onDidChange(() => {
 			if (dropdownIsVisible) {
-				const actions = getOutlineToolbarActions(menu, { notebookEditor: this._editor, outlineEntry: entry });
+				const actions = getOutlineToolbarActions(menu, { notebookEditor: editor, outlineEntry: entry });
 				deferredUpdate = () => toolbar.setActions(actions.primary, actions.secondary);
 
 				return;
 			}
 
-			const actions = getOutlineToolbarActions(menu, { notebookEditor: this._editor, outlineEntry: entry });
+			const actions = getOutlineToolbarActions(menu, { notebookEditor: editor, outlineEntry: entry });
 			toolbar.setActions(actions.primary, actions.secondary);
 		}));
 
@@ -249,9 +253,8 @@ class NotebookOutlineRenderer implements ITreeRenderer<OutlineEntry, FuzzyScore,
 	}
 }
 
-function getOutlineToolbarActions(menu: IMenu, args?: NotebookSectionArgs): { primary: IAction[]; secondary: IAction[] } {
-	// TODO: @Yoyokrazy bring the "inline" back when there's an appropriate run in section icon
-	return getActionBarActions(menu.getActions({ shouldForwardArgs: true, arg: args })); //, g => /^inline/.test(g));
+function getOutlineToolbarActions(menu: IMenu, args?: NotebookOutlineEntryArgs): { primary: IAction[]; secondary: IAction[] } {
+	return getActionBarActions(menu.getActions({ shouldForwardArgs: true, arg: args }), g => /^inline/.test(g));
 }
 
 class NotebookOutlineAccessibility implements IListAccessibilityProvider<OutlineEntry> {
@@ -343,6 +346,28 @@ export class NotebookQuickPickProvider implements IQuickPickDataSource<OutlineEn
 	}
 }
 
+/**
+ * Checks if the given outline entry should be filtered out of the outlinePane
+ *
+ * @param entry the OutlineEntry to check
+ * @param showMarkdownHeadersOnly whether to show only markdown headers
+ * @param showCodeCells whether to show code cells
+ * @param showCodeCellSymbols whether to show code cell symbols
+ * @returns true if the entry should be filtered out of the outlinePane, false if the entry should be visible.
+ */
+function filterEntry(entry: OutlineEntry, showMarkdownHeadersOnly: boolean, showCodeCells: boolean, showCodeCellSymbols: boolean): boolean {
+	// if any are true, return true, this entry should NOT be included in the outline
+	if (
+		(showMarkdownHeadersOnly && entry.cell.cellKind === CellKind.Markup && entry.level === NotebookOutlineConstants.NonHeaderOutlineLevel) ||	// show headers only   + cell is mkdn + is level 7 (not header)
+		(!showCodeCells && entry.cell.cellKind === CellKind.Code) ||																				// show code cells off + cell is code
+		(!showCodeCellSymbols && entry.cell.cellKind === CellKind.Code && entry.level > NotebookOutlineConstants.NonHeaderOutlineLevel)				// show symbols off    + cell is code + is level >7 (nb symbol levels)
+	) {
+		return true;
+	}
+
+	return false;
+}
+
 export class NotebookOutlinePaneProvider implements IDataSource<NotebookCellOutline, OutlineEntry> {
 
 	private readonly _disposables = new DisposableStore();
@@ -378,14 +403,14 @@ export class NotebookOutlinePaneProvider implements IDataSource<NotebookCellOutl
 			return undefined;
 		}
 
-		if (!this.filterEntry(newActive)) {
+		if (!filterEntry(newActive, this.showMarkdownHeadersOnly, this.showCodeCells, this.showCodeCellSymbols)) {
 			return newActive;
 		}
 
 		// find a valid parent
 		let parent = newActive.parent;
 		while (parent) {
-			if (this.filterEntry(parent)) {
+			if (filterEntry(parent, this.showMarkdownHeadersOnly, this.showCodeCells, this.showCodeCellSymbols)) {
 				parent = parent.parent;
 			} else {
 				return parent;
@@ -396,25 +421,6 @@ export class NotebookOutlinePaneProvider implements IDataSource<NotebookCellOutl
 		return undefined;
 	}
 
-	/**
-	 * Checks if the given outline entry should be filtered out of the outlinePane
-	 *
-	 * @param entry the OutlineEntry to check
-	 * @returns true if the entry should be filtered out of the outlinePane
-	 */
-	private filterEntry(entry: OutlineEntry): boolean {
-		// if any are true, return true, this entry should NOT be included in the outline
-		if (
-			(this.showMarkdownHeadersOnly && entry.cell.cellKind === CellKind.Markup && entry.level === NotebookOutlineConstants.NonHeaderOutlineLevel) ||	// show headers only   + cell is mkdn + is level 7 (not header)
-			(!this.showCodeCells && entry.cell.cellKind === CellKind.Code) ||																				// show code cells off + cell is code
-			(!this.showCodeCellSymbols && entry.cell.cellKind === CellKind.Code && entry.level > NotebookOutlineConstants.NonHeaderOutlineLevel)				// show symbols off    + cell is code + is level >7 (nb symbol levels)
-		) {
-			return true;
-		}
-
-		return false;
-	}
-
 	*getChildren(element: NotebookCellOutline | OutlineEntry): Iterable<OutlineEntry> {
 		const isOutline = element instanceof NotebookCellOutline;
 		const entries = isOutline ? this.outlineDataSourceRef?.object?.entries ?? [] : element.children;
@@ -515,7 +521,9 @@ export class NotebookCellOutline implements IOutline<OutlineEntry> {
 	private _breadcrumbsDataSource!: IBreadcrumbsDataSource<OutlineEntry>;
 
 	// view settings
+	private outlineShowCodeCells: boolean;
 	private outlineShowCodeCellSymbols: boolean;
+	private outlineShowMarkdownHeadersOnly: boolean;
 
 	// getters
 	get activeElement(): OutlineEntry | undefined {
@@ -535,7 +543,13 @@ export class NotebookCellOutline implements IOutline<OutlineEntry> {
 		return this._outlineDataSourceReference?.object?.uri;
 	}
 	get isEmpty(): boolean {
-		return this._outlineDataSourceReference?.object?.isEmpty ?? true;
+		if (!this._outlineDataSourceReference?.object?.entries) {
+			return true;
+		}
+
+		return !this._outlineDataSourceReference.object.entries.some(entry => {
+			return !filterEntry(entry, this.outlineShowMarkdownHeadersOnly, this.outlineShowCodeCells, this.outlineShowCodeCellSymbols);
+		});
 	}
 
 	private checkDelayer() {
@@ -555,7 +569,9 @@ export class NotebookCellOutline implements IOutline<OutlineEntry> {
 		@ILanguageFeaturesService private readonly _languageFeaturesService: ILanguageFeaturesService,
 		@INotebookExecutionStateService private readonly _notebookExecutionStateService: INotebookExecutionStateService,
 	) {
+		this.outlineShowCodeCells = this._configurationService.getValue<boolean>(NotebookSetting.outlineShowCodeCells);
 		this.outlineShowCodeCellSymbols = this._configurationService.getValue<boolean>(NotebookSetting.outlineShowCodeCellSymbols);
+		this.outlineShowMarkdownHeadersOnly = this._configurationService.getValue<boolean>(NotebookSetting.outlineShowMarkdownHeadersOnly);
 
 		this.initializeOutline();
 
@@ -612,6 +628,10 @@ export class NotebookCellOutline implements IOutline<OutlineEntry> {
 				e.affectsConfiguration(NotebookSetting.outlineShowCodeCellSymbols) ||
 				e.affectsConfiguration(NotebookSetting.breadcrumbsShowCodeCells)
 			) {
+				this.outlineShowCodeCells = this._configurationService.getValue<boolean>(NotebookSetting.outlineShowCodeCells);
+				this.outlineShowCodeCellSymbols = this._configurationService.getValue<boolean>(NotebookSetting.outlineShowCodeCellSymbols);
+				this.outlineShowMarkdownHeadersOnly = this._configurationService.getValue<boolean>(NotebookSetting.outlineShowMarkdownHeadersOnly);
+
 				this.delayedRecomputeState();
 			}
 		}));
