@@ -4,7 +4,7 @@
  *--------------------------------------------------------------------------------------------*/
 
 import { Event } from '../../../../base/common/event.js';
-import { KeyCode, KeyMod } from '../../../../base/common/keyCodes.js';
+import { KeyChord, KeyCode, KeyMod } from '../../../../base/common/keyCodes.js';
 import { Disposable } from '../../../../base/common/lifecycle.js';
 import { ResourceMap } from '../../../../base/common/map.js';
 import { parse } from '../../../../base/common/marshalling.js';
@@ -19,7 +19,7 @@ import { localize2 } from '../../../../nls.js';
 import { AccessibleViewRegistry } from '../../../../platform/accessibility/browser/accessibleViewRegistry.js';
 import { Action2, MenuId, registerAction2 } from '../../../../platform/actions/common/actions.js';
 import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
-import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
+import { ContextKeyExpr, IContextKeyService } from '../../../../platform/contextkey/common/contextkey.js';
 import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
 import { IInstantiationService, ServicesAccessor } from '../../../../platform/instantiation/common/instantiation.js';
 import { KeybindingsRegistry, KeybindingWeight } from '../../../../platform/keybinding/common/keybindingsRegistry.js';
@@ -43,13 +43,15 @@ import { NOTEBOOK_EDITOR_WIDGET_ACTION_WEIGHT } from '../../notebook/browser/con
 import { INotebookEditorOptions } from '../../notebook/browser/notebookBrowser.js';
 import { NotebookEditorWidget } from '../../notebook/browser/notebookEditorWidget.js';
 import * as icons from '../../notebook/browser/notebookIcons.js';
+import { ReplEditorAccessibleView } from '../../notebook/browser/replEditorAccessibleView.js';
 import { INotebookEditorService } from '../../notebook/browser/services/notebookEditorService.js';
 import { CellEditType, CellKind, NotebookSetting, NotebookWorkingCopyTypeIdentifier, REPL_EDITOR_ID } from '../../notebook/common/notebookCommon.js';
+import { IS_COMPOSITE_NOTEBOOK, MOST_RECENT_REPL_EDITOR, NOTEBOOK_CELL_LIST_FOCUSED, NOTEBOOK_EDITOR_FOCUSED } from '../../notebook/common/notebookContextKeys.js';
 import { NotebookEditorInputOptions } from '../../notebook/common/notebookEditorInput.js';
 import { INotebookEditorModelResolverService } from '../../notebook/common/notebookEditorModelResolverService.js';
 import { INotebookService } from '../../notebook/common/notebookService.js';
 import { isReplEditorControl, ReplEditor, ReplEditorControl } from './replEditor.js';
-import { ReplEditorAccessibilityHelp } from './replEditorAccessibilityHelp.js';
+import { ReplEditorHistoryAccessibilityHelp, ReplEditorInputAccessibilityHelp } from './replEditorAccessibilityHelp.js';
 import { ReplEditorInput } from './replEditorInput.js';
 
 type SerializedNotebookEditorData = { resource: URI; preferredResource: URI; viewType: string; options?: NotebookEditorInputOptions; label?: string };
@@ -222,7 +224,112 @@ class ReplWindowWorkingCopyEditorHandler extends Disposable implements IWorkbenc
 registerWorkbenchContribution2(ReplWindowWorkingCopyEditorHandler.ID, ReplWindowWorkingCopyEditorHandler, WorkbenchPhase.BlockRestore);
 registerWorkbenchContribution2(ReplDocumentContribution.ID, ReplDocumentContribution, WorkbenchPhase.BlockRestore);
 
-AccessibleViewRegistry.register(new ReplEditorAccessibilityHelp());
+AccessibleViewRegistry.register(new ReplEditorInputAccessibilityHelp());
+AccessibleViewRegistry.register(new ReplEditorHistoryAccessibilityHelp());
+
+registerAction2(class extends Action2 {
+	constructor() {
+		super({
+			id: 'repl.focusLastItemExecuted',
+			title: localize2('repl.focusLastReplOutput', 'Focus Most Recent REPL Execution'),
+			category: 'REPL',
+			menu: {
+				id: MenuId.CommandPalette,
+				when: MOST_RECENT_REPL_EDITOR,
+			},
+			keybinding: [{
+				primary: KeyChord(KeyMod.Alt | KeyCode.End, KeyMod.Alt | KeyCode.End),
+				weight: NOTEBOOK_EDITOR_WIDGET_ACTION_WEIGHT,
+				when: ContextKeyExpr.or(IS_COMPOSITE_NOTEBOOK, NOTEBOOK_CELL_LIST_FOCUSED.negate())
+			}],
+			precondition: MOST_RECENT_REPL_EDITOR
+		});
+	}
+
+	async run(accessor: ServicesAccessor, context?: UriComponents): Promise<void> {
+		const editorService = accessor.get(IEditorService);
+		const editorControl = editorService.activeEditorPane?.getControl();
+		const contextKeyService = accessor.get(IContextKeyService);
+
+		let notebookEditor: NotebookEditorWidget | undefined;
+		if (editorControl && isReplEditorControl(editorControl)) {
+			notebookEditor = editorControl.notebookEditor;
+		} else {
+			const uriString = MOST_RECENT_REPL_EDITOR.getValue(contextKeyService);
+			const uri = uriString ? URI.parse(uriString) : undefined;
+
+			if (!uri) {
+				return;
+			}
+			const replEditor = editorService.findEditors(uri)[0];
+
+			if (replEditor) {
+				const editor = await editorService.openEditor(replEditor.editor, replEditor.groupId);
+				const editorControl = editor?.getControl();
+
+				if (editorControl && isReplEditorControl(editorControl)) {
+					notebookEditor = editorControl.notebookEditor;
+				}
+			}
+		}
+
+		const viewModel = notebookEditor?.getViewModel();
+		if (notebookEditor && viewModel) {
+			// last cell of the viewmodel is the last cell history
+			const lastCellIndex = viewModel.length - 1;
+			if (lastCellIndex >= 0) {
+				const cell = viewModel.viewCells[lastCellIndex];
+				notebookEditor.focusNotebookCell(cell, 'container');
+			}
+		}
+	}
+});
+
+registerAction2(class extends Action2 {
+	constructor() {
+		super({
+			id: 'repl.input.focus',
+			title: localize2('repl.input.focus', 'Focus Input Editor'),
+			category: 'REPL',
+			menu: {
+				id: MenuId.CommandPalette,
+				when: MOST_RECENT_REPL_EDITOR,
+			},
+			keybinding: [{
+				when: ContextKeyExpr.and(IS_COMPOSITE_NOTEBOOK, NOTEBOOK_EDITOR_FOCUSED),
+				weight: NOTEBOOK_EDITOR_WIDGET_ACTION_WEIGHT,
+				primary: KeyMod.CtrlCmd | KeyCode.DownArrow
+			}, {
+				when: ContextKeyExpr.and(MOST_RECENT_REPL_EDITOR),
+				weight: KeybindingWeight.WorkbenchContrib + 5,
+				primary: KeyChord(KeyMod.Alt | KeyCode.Home, KeyMod.Alt | KeyCode.Home),
+			}]
+		});
+	}
+
+	async run(accessor: ServicesAccessor): Promise<void> {
+		const editorService = accessor.get(IEditorService);
+		const editorControl = editorService.activeEditorPane?.getControl();
+		const contextKeyService = accessor.get(IContextKeyService);
+
+		if (editorControl && isReplEditorControl(editorControl) && editorControl.notebookEditor) {
+			editorService.activeEditorPane?.focus();
+		}
+		else {
+			const uriString = MOST_RECENT_REPL_EDITOR.getValue(contextKeyService);
+			const uri = uriString ? URI.parse(uriString) : undefined;
+
+			if (!uri) {
+				return;
+			}
+			const replEditor = editorService.findEditors(uri)[0];
+
+			if (replEditor) {
+				await editorService.openEditor({ resource: uri, options: { preserveFocus: false } }, replEditor.groupId);
+			}
+		}
+	}
+});
 
 registerAction2(class extends Action2 {
 	constructor() {
@@ -231,11 +338,15 @@ registerAction2(class extends Action2 {
 			title: localize2('repl.execute', 'Execute REPL input'),
 			category: 'REPL',
 			keybinding: [{
-				when: ContextKeyExpr.equals('activeEditor', 'workbench.editor.repl'),
+				when: ContextKeyExpr.and(
+					IS_COMPOSITE_NOTEBOOK,
+					ContextKeyExpr.equals('activeEditor', 'workbench.editor.repl')
+				),
 				primary: KeyMod.CtrlCmd | KeyCode.Enter,
 				weight: NOTEBOOK_EDITOR_WIDGET_ACTION_WEIGHT
 			}, {
 				when: ContextKeyExpr.and(
+					IS_COMPOSITE_NOTEBOOK,
 					ContextKeyExpr.equals('activeEditor', 'workbench.editor.repl'),
 					ContextKeyExpr.equals('config.interactiveWindow.executeWithShiftEnter', true)
 				),
@@ -243,6 +354,7 @@ registerAction2(class extends Action2 {
 				weight: NOTEBOOK_EDITOR_WIDGET_ACTION_WEIGHT
 			}, {
 				when: ContextKeyExpr.and(
+					IS_COMPOSITE_NOTEBOOK,
 					ContextKeyExpr.equals('activeEditor', 'workbench.editor.repl'),
 					ContextKeyExpr.equals('config.interactiveWindow.executeWithShiftEnter', false)
 				),
@@ -368,6 +480,8 @@ async function executeReplInput(
 	}
 }
 
+AccessibleViewRegistry.register(new ReplEditorAccessibleView());
+
 KeybindingsRegistry.registerCommandAndKeybindingRule({
 	id: 'list.find.replInputFocus',
 	weight: KeybindingWeight.WorkbenchContrib + 1,
