@@ -10,7 +10,6 @@ import { Emitter, Event } from '../../../../../base/common/event.js';
 import { combinedDisposable, Disposable, MutableDisposable } from '../../../../../base/common/lifecycle.js';
 import { sep } from '../../../../../base/common/path.js';
 import { commonPrefixLength } from '../../../../../base/common/strings.js';
-import { ThemeIcon } from '../../../../../base/common/themables.js';
 import { editorSuggestWidgetSelectedBackground } from '../../../../../editor/contrib/suggest/browser/suggestWidget.js';
 import { IConfigurationService } from '../../../../../platform/configuration/common/configuration.js';
 import { IContextKey } from '../../../../../platform/contextkey/common/contextkey.js';
@@ -18,78 +17,21 @@ import { IInstantiationService } from '../../../../../platform/instantiation/com
 import { IStorageService, StorageScope, StorageTarget } from '../../../../../platform/storage/common/storage.js';
 import { TerminalCapability, type ITerminalCapabilityStore } from '../../../../../platform/terminal/common/capabilities/capabilities.js';
 import type { IPromptInputModel, IPromptInputModelState } from '../../../../../platform/terminal/common/capabilities/commandDetection/promptInputModel.js';
-import { ShellIntegrationOscPs } from '../../../../../platform/terminal/common/xterm/shellIntegrationAddon.js';
 import { getListStyles } from '../../../../../platform/theme/browser/defaultStyles.js';
 import { activeContrastBorder } from '../../../../../platform/theme/common/colorRegistry.js';
 import { ITerminalConfigurationService } from '../../../terminal/browser/terminal.js';
 import type { IXtermCore } from '../../../terminal/browser/xterm-private.js';
 import { TerminalStorageKeys } from '../../../terminal/common/terminalStorageKeys.js';
 import { terminalSuggestConfigSection, type ITerminalSuggestConfiguration } from '../common/terminalSuggestConfiguration.js';
-import { SimpleCompletionItem, type ISimpleCompletion } from '../../../../services/suggest/browser/simpleCompletionItem.js';
+import { SimpleCompletionItem } from '../../../../services/suggest/browser/simpleCompletionItem.js';
 import { LineContext, SimpleCompletionModel } from '../../../../services/suggest/browser/simpleCompletionModel.js';
 import { ISimpleSelectedSuggestion, SimpleSuggestWidget } from '../../../../services/suggest/browser/simpleSuggestWidget.js';
 import type { ISimpleSuggestWidgetFontInfo } from '../../../../services/suggest/browser/simpleSuggestWidgetRenderer.js';
-
-export const enum VSCodeSuggestOscPt {
-	Completions = 'Completions',
-	CompletionsPwshCommands = 'CompletionsPwshCommands',
-	CompletionsBash = 'CompletionsBash',
-	CompletionsBashFirstWord = 'CompletionsBashFirstWord'
-}
-
-export type CompressedPwshCompletion = [
-	completionText: string,
-	resultType: number,
-	toolTip?: string,
-	customIcon?: string
-];
-
-export type PwshCompletion = {
-	CompletionText: string;
-	ResultType: number;
-	ToolTip?: string;
-	CustomIcon?: string;
-};
-
-
-/**
- * A map of the pwsh result type enum's value to the corresponding icon to use in completions.
- *
- * | Value | Name              | Description
- * |-------|-------------------|------------
- * | 0     | Text              | An unknown result type, kept as text only
- * | 1     | History           | A history result type like the items out of get-history
- * | 2     | Command           | A command result type like the items out of get-command
- * | 3     | ProviderItem      | A provider item
- * | 4     | ProviderContainer | A provider container
- * | 5     | Property          | A property result type like the property items out of get-member
- * | 6     | Method            | A method result type like the method items out of get-member
- * | 7     | ParameterName     | A parameter name result type like the Parameters property out of get-command items
- * | 8     | ParameterValue    | A parameter value result type
- * | 9     | Variable          | A variable result type like the items out of get-childitem variable:
- * | 10    | Namespace         | A namespace
- * | 11    | Type              | A type name
- * | 12    | Keyword           | A keyword
- * | 13    | DynamicKeyword    | A dynamic keyword
- *
- * @see https://docs.microsoft.com/en-us/dotnet/api/system.management.automation.completionresulttype?view=powershellsdk-7.0.0
- */
-const pwshTypeToIconMap: { [type: string]: ThemeIcon | undefined } = {
-	0: Codicon.symbolText,
-	1: Codicon.history,
-	2: Codicon.symbolMethod,
-	3: Codicon.symbolFile,
-	4: Codicon.folder,
-	5: Codicon.symbolProperty,
-	6: Codicon.symbolMethod,
-	7: Codicon.symbolVariable,
-	8: Codicon.symbolValue,
-	9: Codicon.symbolVariable,
-	10: Codicon.symbolNamespace,
-	11: Codicon.symbolInterface,
-	12: Codicon.symbolKeyword,
-	13: Codicon.symbolKeyword
-};
+import { ITerminalCompletion, ITerminalCompletionService, TerminalCompletionItemKind } from './terminalCompletionService.js';
+import { TerminalShellType } from '../../../../../platform/terminal/common/terminal.js';
+import { CancellationToken, CancellationTokenSource } from '../../../../../base/common/cancellation.js';
+import { IExtensionService } from '../../../../services/extensions/common/extensions.js';
+import { ThemeIcon } from '../../../../../base/common/themables.js';
 
 export interface ISuggestController {
 	isPasting: boolean;
@@ -100,7 +42,6 @@ export interface ISuggestController {
 	acceptSelectedSuggestion(suggestion?: Pick<ISimpleSelectedSuggestion, 'item' | 'model'>): void;
 	hideSuggestWidget(): void;
 }
-
 export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggestController {
 	private _terminal?: Terminal;
 
@@ -117,45 +58,53 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 	private _enableWidget: boolean = true;
 	private _pathSeparator: string = sep;
 	private _isFilteringDirectories: boolean = false;
-	private _mostRecentCompletion?: ISimpleCompletion;
-
-	private _codeCompletionsRequested: boolean = false;
-	private _gitCompletionsRequested: boolean = false;
+	private _mostRecentCompletion?: ITerminalCompletion;
 
 	// TODO: Remove these in favor of prompt input state
 	private _leadingLineContent?: string;
 	private _cursorIndexDelta: number = 0;
+	private _requestedCompletionsIndex: number = 0;
 
-	private _lastUserDataTimestamp: number = 0;
-	private _lastAcceptedCompletionTimestamp: number = 0;
 	private _lastUserData?: string;
+	static lastAcceptedCompletionTimestamp: number = 0;
+	private _lastUserDataTimestamp: number = 0;
+
+	private _cancellationTokenSource: CancellationTokenSource | undefined;
 
 	isPasting: boolean = false;
-
-	static requestCompletionsSequence = '\x1b[24~e'; // F12,e
-	static requestGlobalCompletionsSequence = '\x1b[24~f'; // F12,f
-	static requestEnableGitCompletionsSequence = '\x1b[24~g'; // F12,g
-	static requestEnableCodeCompletionsSequence = '\x1b[24~h'; // F12,h
+	shellType: TerminalShellType | undefined;
 
 	private readonly _onBell = this._register(new Emitter<void>());
 	readonly onBell = this._onBell.event;
 	private readonly _onAcceptedCompletion = this._register(new Emitter<string>());
 	readonly onAcceptedCompletion = this._onAcceptedCompletion.event;
-	private readonly _onDidRequestCompletions = this._register(new Emitter<void>());
-	readonly onDidRequestCompletions = this._onDidRequestCompletions.event;
 	private readonly _onDidReceiveCompletions = this._register(new Emitter<void>());
 	readonly onDidReceiveCompletions = this._onDidReceiveCompletions.event;
 
+	private _kindToIconMap = new Map<number, ThemeIcon>([
+		[TerminalCompletionItemKind.File, Codicon.file],
+		[TerminalCompletionItemKind.Folder, Codicon.folder],
+		[TerminalCompletionItemKind.Flag, Codicon.symbolProperty],
+		[TerminalCompletionItemKind.Method, Codicon.symbolMethod],
+		[TerminalCompletionItemKind.Argument, Codicon.symbolVariable]
+	]);
+
+	private _shouldSyncWhenReady: boolean = false;
+
 	constructor(
-		private readonly _cachedPwshCommands: Set<SimpleCompletionItem>,
+		shellType: TerminalShellType | undefined,
 		private readonly _capabilities: ITerminalCapabilityStore,
 		private readonly _terminalSuggestWidgetVisibleContextKey: IContextKey<boolean>,
+		@ITerminalCompletionService private readonly _terminalCompletionService: ITerminalCompletionService,
 		@IConfigurationService private readonly _configurationService: IConfigurationService,
 		@IInstantiationService private readonly _instantiationService: IInstantiationService,
 		@ITerminalConfigurationService private readonly _terminalConfigurationService: ITerminalConfigurationService,
+		@IExtensionService private readonly _extensionService: IExtensionService
 	) {
 		super();
 
+		this.shellType = shellType;
+
 		this._register(Event.runAndSubscribe(Event.any(
 			this._capabilities.onDidAddCapabilityType,
 			this._capabilities.onDidRemoveCapabilityType
@@ -168,7 +117,12 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 						this._promptInputModel.onDidChangeInput(e => this._sync(e)),
 						this._promptInputModel.onDidFinishInput(() => this.hideSuggestWidget()),
 					);
+					if (this._shouldSyncWhenReady) {
+						this._sync(this._promptInputModel);
+						this._shouldSyncWhenReady = false;
+					}
 				}
+				this._register(commandDetection.onCommandExecuted(() => this.hideSuggestWidget()));
 			} else {
 				this._promptInputModel = undefined;
 			}
@@ -177,15 +131,101 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 
 	activate(xterm: Terminal): void {
 		this._terminal = xterm;
-		this._register(xterm.parser.registerOscHandler(ShellIntegrationOscPs.VSCode, data => {
-			return this._handleVSCodeSequence(data);
-		}));
-		this._register(xterm.onData(e => {
+		this._register(xterm.onData(async e => {
 			this._lastUserData = e;
 			this._lastUserDataTimestamp = Date.now();
 		}));
 	}
 
+	private async _handleCompletionProviders(terminal: Terminal | undefined, token: CancellationToken, explicitlyInvoked?: boolean): Promise<void> {
+		// Nothing to handle if the terminal is not attached
+		if (!terminal?.element || !this._enableWidget || !this._promptInputModel) {
+			return;
+		}
+
+		// Only show the suggest widget if the terminal is focused
+		if (!dom.isAncestorOfActiveElement(terminal.element)) {
+			return;
+		}
+
+		if (!this.shellType) {
+			return;
+		}
+
+		let doNotRequestExtensionCompletions = false;
+		// Ensure that a key has been pressed since the last accepted completion in order to prevent
+		// completions being requested again right after accepting a completion
+		if (this._lastUserDataTimestamp < SuggestAddon.lastAcceptedCompletionTimestamp) {
+			doNotRequestExtensionCompletions = true;
+		}
+
+		const enableExtensionCompletions = this._configurationService.getValue<ITerminalSuggestConfiguration>(terminalSuggestConfigSection).enableExtensionCompletions;
+		if (enableExtensionCompletions && !doNotRequestExtensionCompletions) {
+			await this._extensionService.activateByEvent('onTerminalCompletionsRequested');
+		}
+
+		const providedCompletions = await this._terminalCompletionService.provideCompletions(this._promptInputModel.prefix, this._promptInputModel.cursorIndex, this.shellType, token, doNotRequestExtensionCompletions);
+		if (!providedCompletions?.length || token.isCancellationRequested) {
+			return;
+		}
+		this._onDidReceiveCompletions.fire();
+
+		this._requestedCompletionsIndex = this._promptInputModel.cursorIndex;
+
+		this._currentPromptInputState = {
+			value: this._promptInputModel.value,
+			prefix: this._promptInputModel.prefix,
+			suffix: this._promptInputModel.suffix,
+			cursorIndex: this._promptInputModel.cursorIndex,
+			ghostTextIndex: this._promptInputModel.ghostTextIndex
+		};
+
+		this._leadingLineContent = this._currentPromptInputState.prefix.substring(0, this._requestedCompletionsIndex + this._cursorIndexDelta);
+
+		const completions = providedCompletions.flat();
+		if (!completions?.length) {
+			return;
+		}
+
+		const firstChar = this._leadingLineContent.length === 0 ? '' : this._leadingLineContent[0];
+		// This is a TabExpansion2 result
+		if (this._leadingLineContent.includes(' ') || firstChar === '[') {
+			this._leadingLineContent = this._promptInputModel.prefix;
+		}
+
+		if (this._mostRecentCompletion?.isDirectory && completions.every(e => e.isDirectory)) {
+			completions.push(this._mostRecentCompletion);
+		}
+		this._mostRecentCompletion = undefined;
+
+		this._cursorIndexDelta = this._currentPromptInputState.cursorIndex - this._requestedCompletionsIndex;
+
+		let normalizedLeadingLineContent = this._leadingLineContent;
+
+		// If there is a single directory in the completions:
+		// - `\` and `/` are normalized such that either can be used
+		// - Using `\` or `/` will request new completions. It's important that this only occurs
+		//   when a directory is present, if not completions like git branches could be requested
+		//   which leads to flickering
+		this._isFilteringDirectories = completions.some(e => e.isDirectory);
+		if (this._isFilteringDirectories) {
+			const firstDir = completions.find(e => e.isDirectory);
+			this._pathSeparator = firstDir?.label.match(/(?<sep>[\\\/])/)?.groups?.sep ?? sep;
+			normalizedLeadingLineContent = normalizePathSeparator(normalizedLeadingLineContent, this._pathSeparator);
+		}
+		for (const completion of completions) {
+			if (!completion.icon && completion.kind !== undefined) {
+				completion.icon = this._kindToIconMap.get(completion.kind);
+			}
+		}
+		const lineContext = new LineContext(normalizedLeadingLineContent, this._cursorIndexDelta);
+		const model = new SimpleCompletionModel(completions.filter(c => !!c.label).map(c => new SimpleCompletionItem(c)), lineContext);
+		if (token.isCancellationRequested) {
+			return;
+		}
+		this._showCompletions(model, explicitlyInvoked);
+	}
+
 	setContainerWithOverflow(container: HTMLElement): void {
 		this._container = container;
 	}
@@ -194,45 +234,30 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 		this._screen = screen;
 	}
 
-	private _requestCompletions(): void {
+	resetWidgetSize(): void {
+		this._suggestWidget?.resetWidgetSize();
+	}
+
+	async requestCompletions(explicitlyInvoked?: boolean): Promise<void> {
 		if (!this._promptInputModel) {
+			this._shouldSyncWhenReady = true;
 			return;
 		}
 
 		if (this.isPasting) {
 			return;
 		}
-
-		const builtinCompletionsConfig = this._configurationService.getValue<ITerminalSuggestConfiguration>(terminalSuggestConfigSection).builtinCompletions;
-		if (!this._codeCompletionsRequested && builtinCompletionsConfig.pwshCode) {
-			this._onAcceptedCompletion.fire(SuggestAddon.requestEnableCodeCompletionsSequence);
-			this._codeCompletionsRequested = true;
+		if (this._cancellationTokenSource) {
+			this._cancellationTokenSource.cancel();
+			this._cancellationTokenSource.dispose();
 		}
-		if (!this._gitCompletionsRequested && builtinCompletionsConfig.pwshGit) {
-			this._onAcceptedCompletion.fire(SuggestAddon.requestEnableGitCompletionsSequence);
-			this._gitCompletionsRequested = true;
-		}
-
-		// Request global completions if there are none cached
-		if (this._cachedPwshCommands.size === 0) {
-			this._requestGlobalCompletions();
-		}
-
-		// Ensure that a key has been pressed since the last accepted completion in order to prevent
-		// completions being requested again right after accepting a completion
-		if (this._lastUserDataTimestamp > this._lastAcceptedCompletionTimestamp) {
-			this._onAcceptedCompletion.fire(SuggestAddon.requestCompletionsSequence);
-			this._onDidRequestCompletions.fire();
-		}
-	}
-
-	private _requestGlobalCompletions(): void {
-		this._onAcceptedCompletion.fire(SuggestAddon.requestGlobalCompletionsSequence);
+		this._cancellationTokenSource = new CancellationTokenSource();
+		const token = this._cancellationTokenSource.token;
+		await this._handleCompletionProviders(this._terminal, token, explicitlyInvoked);
 	}
 
 	private _sync(promptInputState: IPromptInputModelState): void {
 		const config = this._configurationService.getValue<ITerminalSuggestConfiguration>(terminalSuggestConfigSection);
-
 		if (!this._mostRecentPromptInputState || promptInputState.cursorIndex > this._mostRecentPromptInputState.cursorIndex) {
 			// If input has been added
 			let sent = false;
@@ -240,11 +265,14 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 			// Quick suggestions
 			if (!this._terminalSuggestWidgetVisibleContextKey.get()) {
 				if (config.quickSuggestions) {
+					// TODO: Make the regex code generic
+					// TODO: Don't use `\[` in bash/zsh
+					// If first character or first character after a space (or `[` in pwsh), request completions
 					if (promptInputState.cursorIndex === 1 || promptInputState.prefix.match(/([\s\[])[^\s]$/)) {
 						// Never request completions if the last key sequence was up or down as the user was likely
 						// navigating history
 						if (!this._lastUserData?.match(/^\x1b[\[O]?[A-D]$/)) {
-							this._requestCompletions();
+							this.requestCompletions();
 							sent = true;
 						}
 					}
@@ -262,9 +290,23 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 					// with git branches in particular
 					this._isFilteringDirectories && prefix?.match(/[\\\/]$/)
 				) {
-					this._requestCompletions();
+					this.requestCompletions();
 					sent = true;
 				}
+				if (!sent) {
+					for (const provider of this._terminalCompletionService.providers) {
+						if (!provider.triggerCharacters) {
+							continue;
+						}
+						for (const char of provider.triggerCharacters) {
+							if (prefix?.endsWith(char)) {
+								this.requestCompletions();
+								sent = true;
+								break;
+							}
+						}
+					}
+				}
 			}
 		}
 
@@ -283,14 +325,15 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 
 		// Hide the widget if the cursor moves to the left of the initial position as the
 		// completions are no longer valid
-		if (this._currentPromptInputState.cursorIndex < this._replacementIndex + this._replacementLength) {
+		// to do: get replacement length to be correct, readd this?
+		if (this._currentPromptInputState && this._currentPromptInputState.cursorIndex <= this._leadingLineContent.length) {
 			this.hideSuggestWidget();
 			return;
 		}
 
 		if (this._terminalSuggestWidgetVisibleContextKey.get()) {
-			this._cursorIndexDelta = this._currentPromptInputState.cursorIndex - (this._replacementIndex + this._replacementLength);
-			let normalizedLeadingLineContent = this._currentPromptInputState.value.substring(this._replacementIndex, this._replacementIndex + this._replacementLength + this._cursorIndexDelta);
+			this._cursorIndexDelta = this._currentPromptInputState.cursorIndex - (this._requestedCompletionsIndex);
+			let normalizedLeadingLineContent = this._currentPromptInputState.value.substring(0, this._requestedCompletionsIndex + this._cursorIndexDelta);
 			if (this._isFilteringDirectories) {
 				normalizedLeadingLineContent = normalizePathSeparator(normalizedLeadingLineContent, this._pathSeparator);
 			}
@@ -316,197 +359,6 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 		});
 	}
 
-	private _handleVSCodeSequence(data: string): boolean | Promise<boolean> {
-		if (!this._terminal) {
-			return false;
-		}
-
-		// Pass the sequence along to the capability
-		const [command, ...args] = data.split(';');
-		switch (command) {
-			case VSCodeSuggestOscPt.Completions:
-				this._handleCompletionsSequence(this._terminal, data, command, args);
-				return true;
-			case VSCodeSuggestOscPt.CompletionsBash:
-				this._handleCompletionsBashSequence(this._terminal, data, command, args);
-				return true;
-			case VSCodeSuggestOscPt.CompletionsBashFirstWord:
-				return this._handleCompletionsBashFirstWordSequence(this._terminal, data, command, args);
-		}
-
-		// Unrecognized sequence
-		return false;
-	}
-	private _replacementIndex: number = 0;
-	private _replacementLength: number = 0;
-
-	private _handleCompletionsSequence(terminal: Terminal, data: string, command: string, args: string[]): void {
-		this._onDidReceiveCompletions.fire();
-
-		// Nothing to handle if the terminal is not attached
-		if (!terminal.element || !this._enableWidget || !this._promptInputModel) {
-			return;
-		}
-
-		// Only show the suggest widget if the terminal is focused
-		if (!dom.isAncestorOfActiveElement(terminal.element)) {
-			return;
-		}
-
-		let replacementIndex = 0;
-		let replacementLength = this._promptInputModel.cursorIndex;
-
-		this._currentPromptInputState = {
-			value: this._promptInputModel.value,
-			prefix: this._promptInputModel.prefix,
-			suffix: this._promptInputModel.suffix,
-			cursorIndex: this._promptInputModel.cursorIndex,
-			ghostTextIndex: this._promptInputModel.ghostTextIndex
-		};
-
-		this._leadingLineContent = this._currentPromptInputState.prefix.substring(replacementIndex, replacementIndex + replacementLength + this._cursorIndexDelta);
-
-		const payload = data.slice(command.length + args[0].length + args[1].length + args[2].length + 4/*semi-colons*/);
-		const rawCompletions: PwshCompletion | PwshCompletion[] | CompressedPwshCompletion[] | CompressedPwshCompletion = args.length === 0 || payload.length === 0 ? undefined : JSON.parse(payload);
-		const completions = parseCompletionsFromShell(rawCompletions);
-
-		const firstChar = this._leadingLineContent.length === 0 ? '' : this._leadingLineContent[0];
-		// This is a TabExpansion2 result
-		if (this._leadingLineContent.includes(' ') || firstChar === '[') {
-			replacementIndex = parseInt(args[0]);
-			replacementLength = parseInt(args[1]);
-			this._leadingLineContent = this._promptInputModel.prefix;
-		}
-		// This is a global command, add cached commands list to completions
-		else {
-			completions.push(...this._cachedPwshCommands);
-		}
-
-		this._replacementIndex = replacementIndex;
-		this._replacementLength = replacementLength;
-
-		if (this._mostRecentCompletion?.isDirectory && completions.every(e => e.completion.isDirectory)) {
-			completions.push(new SimpleCompletionItem(this._mostRecentCompletion));
-		}
-		this._mostRecentCompletion = undefined;
-
-		this._cursorIndexDelta = this._currentPromptInputState.cursorIndex - (replacementIndex + replacementLength);
-
-		let normalizedLeadingLineContent = this._leadingLineContent;
-
-		// If there is a single directory in the completions:
-		// - `\` and `/` are normalized such that either can be used
-		// - Using `\` or `/` will request new completions. It's important that this only occurs
-		//   when a directory is present, if not completions like git branches could be requested
-		//   which leads to flickering
-		this._isFilteringDirectories = completions.some(e => e.completion.isDirectory);
-		if (this._isFilteringDirectories) {
-			const firstDir = completions.find(e => e.completion.isDirectory);
-			this._pathSeparator = firstDir?.completion.label.match(/(?<sep>[\\\/])/)?.groups?.sep ?? sep;
-			normalizedLeadingLineContent = normalizePathSeparator(normalizedLeadingLineContent, this._pathSeparator);
-		}
-		const lineContext = new LineContext(normalizedLeadingLineContent, this._cursorIndexDelta);
-		const model = new SimpleCompletionModel(completions, lineContext, replacementIndex, replacementLength);
-		this._handleCompletionModel(model);
-	}
-
-	// TODO: These aren't persisted across reloads
-	// TODO: Allow triggering anywhere in the first word based on the cached completions
-	private _cachedBashAliases: Set<SimpleCompletionItem> = new Set();
-	private _cachedBashBuiltins: Set<SimpleCompletionItem> = new Set();
-	private _cachedBashCommands: Set<SimpleCompletionItem> = new Set();
-	private _cachedBashKeywords: Set<SimpleCompletionItem> = new Set();
-	private _cachedFirstWord?: SimpleCompletionItem[];
-	private _handleCompletionsBashFirstWordSequence(terminal: Terminal, data: string, command: string, args: string[]): boolean {
-		const type = args[0];
-		const completionList: string[] = data.slice(command.length + type.length + 2/*semi-colons*/).split(';');
-		let set: Set<SimpleCompletionItem>;
-		switch (type) {
-			case 'alias': set = this._cachedBashAliases; break;
-			case 'builtin': set = this._cachedBashBuiltins; break;
-			case 'command': set = this._cachedBashCommands; break;
-			case 'keyword': set = this._cachedBashKeywords; break;
-			default: return false;
-		}
-		set.clear();
-		const distinctLabels: Set<string> = new Set();
-		for (const label of completionList) {
-			distinctLabels.add(label);
-		}
-		for (const label of distinctLabels) {
-			set.add(new SimpleCompletionItem({
-				label,
-				icon: Codicon.symbolString,
-				detail: type
-			}));
-		}
-		// Invalidate compound list cache
-		this._cachedFirstWord = undefined;
-		return true;
-	}
-
-	private _handleCompletionsBashSequence(terminal: Terminal, data: string, command: string, args: string[]): void {
-		// Nothing to handle if the terminal is not attached
-		if (!terminal.element) {
-			return;
-		}
-
-		let replacementIndex = parseInt(args[0]);
-		const replacementLength = parseInt(args[1]);
-		if (!args[2]) {
-			this._onBell.fire();
-			return;
-		}
-
-		const completionList: string[] = data.slice(command.length + args[0].length + args[1].length + args[2].length + 4/*semi-colons*/).split(';');
-		// TODO: Create a trigger suggest command which encapsulates sendSequence and uses cached if available
-		let completions: SimpleCompletionItem[];
-		// TODO: This 100 is a hack just for the prototype, this should get it based on some terminal input model
-		if (replacementIndex !== 100 && completionList.length > 0) {
-			completions = completionList.map(label => {
-				return new SimpleCompletionItem({
-					label: label,
-					icon: Codicon.symbolProperty
-				});
-			});
-		} else {
-			replacementIndex = 0;
-			if (!this._cachedFirstWord) {
-				this._cachedFirstWord = [
-					...this._cachedBashAliases,
-					...this._cachedBashBuiltins,
-					...this._cachedBashCommands,
-					...this._cachedBashKeywords
-				];
-				this._cachedFirstWord.sort((a, b) => {
-					const aCode = a.completion.label.charCodeAt(0);
-					const bCode = b.completion.label.charCodeAt(0);
-					const isANonAlpha = aCode < 65 || aCode > 90 && aCode < 97 || aCode > 122 ? 1 : 0;
-					const isBNonAlpha = bCode < 65 || bCode > 90 && bCode < 97 || bCode > 122 ? 1 : 0;
-					if (isANonAlpha !== isBNonAlpha) {
-						return isANonAlpha - isBNonAlpha;
-					}
-					return a.completion.label.localeCompare(b.completion.label);
-				});
-			}
-			completions = this._cachedFirstWord;
-		}
-		if (completions.length === 0) {
-			return;
-		}
-
-		this._leadingLineContent = completions[0].completion.label.slice(0, replacementLength);
-		const model = new SimpleCompletionModel(completions, new LineContext(this._leadingLineContent, replacementIndex), replacementIndex, replacementLength);
-		if (completions.length === 1) {
-			const insertText = completions[0].completion.label.substring(replacementLength);
-			if (insertText.length === 0) {
-				this._onBell.fire();
-				return;
-			}
-		}
-		this._handleCompletionModel(model);
-	}
-
 	private _getTerminalDimensions(): { width: number; height: number } {
 		const cssCellDims = (this._terminal as any as { _core: IXtermCore })._core._renderService.dimensions.css.cell;
 		return {
@@ -515,13 +367,13 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 		};
 	}
 
-	private _handleCompletionModel(model: SimpleCompletionModel): void {
+	private _showCompletions(model: SimpleCompletionModel, explicitlyInvoked?: boolean): void {
 		if (!this._terminal?.element) {
 			return;
 		}
 		const suggestWidget = this._ensureSuggestWidget(this._terminal);
 		suggestWidget.setCompletionModel(model);
-		if (model.items.length === 0 || !this._promptInputModel) {
+		if (!this._promptInputModel || !explicitlyInvoked && model.items.length === 0) {
 			return;
 		}
 		this._model = model;
@@ -530,6 +382,7 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 			return;
 		}
 		const xtermBox = this._screen!.getBoundingClientRect();
+
 		suggestWidget.showSuggestions(0, false, false, {
 			left: xtermBox.left + this._terminal.buffer.active.cursorX * dimensions.width,
 			top: xtermBox.top + this._terminal.buffer.active.cursorY * dimensions.height,
@@ -538,7 +391,6 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 	}
 
 	private _ensureSuggestWidget(terminal: Terminal): SimpleSuggestWidget {
-		this._terminalSuggestWidgetVisibleContextKey.set(true);
 		if (!this._suggestWidget) {
 			const c = this._terminalConfigurationService.config;
 			const font = this._terminalConfigurationService.getFont(dom.getActiveWindow());
@@ -563,6 +415,7 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 			this._register(this._suggestWidget.onDidSelect(async e => this.acceptSelectedSuggestion(e)));
 			this._register(this._suggestWidget.onDidHide(() => this._terminalSuggestWidgetVisibleContextKey.set(false)));
 			this._register(this._suggestWidget.onDidShow(() => this._terminalSuggestWidgetVisibleContextKey.set(true)));
+			this._terminalSuggestWidgetVisibleContextKey.set(false);
 		}
 		return this._suggestWidget;
 	}
@@ -588,10 +441,10 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 			suggestion = this._suggestWidget?.getFocusedItem();
 		}
 		const initialPromptInputState = this._mostRecentPromptInputState;
-		if (!suggestion || !initialPromptInputState || !this._leadingLineContent || !this._model) {
+		if (!suggestion || !initialPromptInputState || this._leadingLineContent === undefined || !this._model) {
 			return;
 		}
-		this._lastAcceptedCompletionTimestamp = Date.now();
+		SuggestAddon.lastAcceptedCompletionTimestamp = Date.now();
 		this._suggestWidget?.hide();
 
 		const currentPromptInputState = this._currentPromptInputState ?? initialPromptInputState;
@@ -599,7 +452,7 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 		// The replacement text is any text after the replacement index for the completions, this
 		// includes any text that was there before the completions were requested and any text added
 		// since to refine the completion.
-		const replacementText = currentPromptInputState.value.substring(this._model.replacementIndex, currentPromptInputState.cursorIndex);
+		const replacementText = currentPromptInputState.value.substring(suggestion.item.completion.replacementIndex, currentPromptInputState.cursorIndex);
 
 		// Right side of replacement text in the same word
 		let rightSideReplacementText = '';
@@ -616,8 +469,11 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 		}
 
 		const completion = suggestion.item.completion;
-		const completionText = completion.label;
-
+		let completionText = completion.label;
+		if ((completion.isDirectory || completion.isFile) && completionText.includes(' ')) {
+			// Escape spaces in files or folders so they're valid paths
+			completionText = completionText.replaceAll(' ', '\\ ');
+		}
 		let runOnEnter = false;
 		if (respectRunOnEnter) {
 			const runOnEnterConfig = this._configurationService.getValue<ITerminalSuggestConfiguration>(terminalSuggestConfigSection).runOnEnter;
@@ -642,14 +498,14 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 
 		// For folders, allow the next completion request to get completions for that folder
 		if (completion.icon === Codicon.folder) {
-			this._lastAcceptedCompletionTimestamp = 0;
+			SuggestAddon.lastAcceptedCompletionTimestamp = 0;
 		}
 
 		this._mostRecentCompletion = completion;
 
-		const commonPrefixLen = commonPrefixLength(replacementText, completion.label);
+		const commonPrefixLen = commonPrefixLength(replacementText, completionText);
 		const commonPrefix = replacementText.substring(replacementText.length - 1 - commonPrefixLen, replacementText.length - 1);
-		const completionSuffix = completion.label.substring(commonPrefixLen);
+		const completionSuffix = completionText.substring(commonPrefixLen);
 		let resultSequence: string;
 		if (currentPromptInputState.suffix.length > 0 && currentPromptInputState.prefix.endsWith(commonPrefix) && currentPromptInputState.suffix.startsWith(completionSuffix)) {
 			// Move right to the end of the completion
@@ -669,11 +525,12 @@ export class SuggestAddon extends Disposable implements ITerminalAddon, ISuggest
 
 		// Send the completion
 		this._onAcceptedCompletion.fire(resultSequence);
-
 		this.hideSuggestWidget();
 	}
 
 	hideSuggestWidget(): void {
+		this._cancellationTokenSource?.cancel();
+		this._cancellationTokenSource = undefined;
 		this._currentPromptInputState = undefined;
 		this._leadingLineContent = undefined;
 		this._suggestWidget?.hide();
@@ -710,89 +567,7 @@ class PersistedWidgetSize {
 		this._storageService.remove(this._key, StorageScope.PROFILE);
 	}
 }
-
-export function parseCompletionsFromShell(rawCompletions: PwshCompletion | PwshCompletion[] | CompressedPwshCompletion[] | CompressedPwshCompletion): SimpleCompletionItem[] {
-	if (!rawCompletions) {
-		return [];
-	}
-	let typedRawCompletions: PwshCompletion[];
-	if (!Array.isArray(rawCompletions)) {
-		typedRawCompletions = [rawCompletions];
-	} else {
-		if (rawCompletions.length === 0) {
-			return [];
-		}
-		if (typeof rawCompletions[0] === 'string') {
-			typedRawCompletions = [rawCompletions as CompressedPwshCompletion].map(e => ({
-				CompletionText: e[0],
-				ResultType: e[1],
-				ToolTip: e[2],
-				CustomIcon: e[3],
-			}));
-		} else if (Array.isArray(rawCompletions[0])) {
-			typedRawCompletions = (rawCompletions as CompressedPwshCompletion[]).map(e => ({
-				CompletionText: e[0],
-				ResultType: e[1],
-				ToolTip: e[2],
-				CustomIcon: e[3],
-			}));
-		} else {
-			typedRawCompletions = rawCompletions as PwshCompletion[];
-		}
-	}
-	return typedRawCompletions.map(e => rawCompletionToSimpleCompletionItem(e));
-}
-
-function rawCompletionToSimpleCompletionItem(rawCompletion: PwshCompletion): SimpleCompletionItem {
-	// HACK: Somewhere along the way from the powershell script to here, the path separator at the
-	// end of directories may go missing, likely because `\"` -> `"`. As a result, make sure there
-	// is a trailing separator at the end of all directory completions. This should not be done for
-	// `.` and `..` entries because they are optimized not for navigating to different directories
-	// but for passing as args.
-	let label = rawCompletion.CompletionText;
-	if (
-		rawCompletion.ResultType === 4 &&
-		!label.match(/^[\-+]$/) && // Don't add a `/` to `-` or `+` (navigate location history)
-		!label.match(/^\.\.?$/) &&
-		!label.match(/[\\\/]$/)
-	) {
-		const separator = label.match(/(?<sep>[\\\/])/)?.groups?.sep ?? sep;
-		label = label + separator;
-	}
-
-	// If tooltip is not present it means it's the same as label
-	const detail = rawCompletion.ToolTip ?? label;
-
-	// Pwsh gives executables a result type of 2, but we want to treat them as files wrt the sorting
-	// and file extension score boost. An example of where this improves the experience is typing
-	// `git`, `git.exe` should appear at the top and beat `git-lfs.exe`. Keep the same icon though.
-	const icon = getIcon(rawCompletion.ResultType, rawCompletion.CustomIcon);
-	const isExecutable = rawCompletion.ResultType === 2 && rawCompletion.CompletionText.match(/\.[a-z0-9]{2,4}$/i);
-	if (isExecutable) {
-		rawCompletion.ResultType = 3;
-	}
-
-	return new SimpleCompletionItem({
-		label,
-		icon,
-		detail,
-		isFile: rawCompletion.ResultType === 3,
-		isDirectory: rawCompletion.ResultType === 4,
-		isKeyword: rawCompletion.ResultType === 12,
-	});
-}
-
-function getIcon(resultType: number, customIconId?: string): ThemeIcon {
-	if (customIconId) {
-		const icon: ThemeIcon | undefined = customIconId in Codicon ? (Codicon as { [id: string]: ThemeIcon | undefined })[customIconId] : Codicon.symbolText;
-		if (icon) {
-			return icon;
-		}
-	}
-	return pwshTypeToIconMap[resultType] ?? Codicon.symbolText;
-}
-
-function normalizePathSeparator(path: string, sep: string): string {
+export function normalizePathSeparator(path: string, sep: string): string {
 	if (sep === '/') {
 		return path.replaceAll('\\', '/');
 	}
